#!/bin/sh -eu
# brmrunas.sh

usage()
{
    cat <<EOF
brmrunas.sh

Starts a BRM command, giving it the power to run some sub-commands as
a different user. You will be prompted for that user's password.

Usage:

./brmrunas.sh username command [param1 param2 ...]

Example:

./brmrunas.sh sassrv restore /BRM/backup1
EOF
    return 0
}

if [ "$#" -lt 2 ]; then
    echo "Missing parameters."
    usage
    exit 1
fi

username="$1"
shift

if [ "$username" == "-h" -o "$username" == "--help" ]; then
    usage
    exit 0
fi

starting_username=`id -u -n`

progdir=`dirname $0`

if [ ! -d "$progdir/temp" ]; then
    mkdir "$progdir/temp"
fi

export BRM_EXEC_FIFO_IN="$progdir/temp/exec-in"
export BRM_EXEC_FIFO_OUT="$progdir/temp/exec-out"

if [ -e $BRM_EXEC_FIFO_IN -o -e $BRM_EXEC_FIFO_OUT ]; then
    echo It appears that brmrunas.sh is already running.
    echo If this is not the case, remove the following two files and try again:
    echo $BRM_EXEC_FIFO_IN
    echo $BRM_EXEC_FIFO_OUT
    exit 1
fi

mkfifo $BRM_EXEC_FIFO_IN
mkfifo $BRM_EXEC_FIFO_OUT
chmod ugo+rw $BRM_EXEC_FIFO_IN
chmod ugo+rw $BRM_EXEC_FIFO_OUT

export BRM_SECURITY_TOKEN=`dd if=/dev/urandom count=1 2> /dev/null | cksum | cut -f1 -d" "`

# Run the executor utility in the background as the specified user
if [ "$username" != "$starting_username" ]; then
    # This will prompt for a password
    su $username -c "$progdir/scripts/executor.sh"
    executor_rc=$?
else
    $progdir/scripts/executor.sh
    executor_rc=$?
fi
if [ "$executor_rc" -ne "0" ]; then
    echo Error starting executor as user $username
    rm -f $BRM_EXEC_FIFO_IN
    rm -f $BRM_EXEC_FIFO_OUT
    exit 1
fi

# Clean up when the script exits for any reason
cleanup()
{
    # Stop the executor
    cat > $BRM_EXEC_FIFO_IN <<EOF
${BRM_SECURITY_TOKEN}
${USER}
quit
EOF

    # Give the executor time to exit
    sleep 2

    # Remove the fifos
    rm -f $BRM_EXEC_FIFO_IN
    rm -f $BRM_EXEC_FIFO_OUT

    # Explicitly exit so script doesn't resume after signal handler
    exit
}

trap cleanup INT TERM EXIT

# Run the BRM command in the foreground
$progdir/brm.sh "$@"

# end-of-file
