package com.sas.solutions.brm.dav;

import java.rmi.RemoteException;

import com.sas.metadata.logical.ServicesUtil;
import com.sas.services.ServiceException;
import com.sas.services.deployment.CorePlatformServices;
import com.sas.services.information.InformationServiceInterface;
import com.sas.services.session.SessionContextInterface;
import com.sas.services.user.UserContextInterface;
import com.sas.services.user.UserServiceInterface;


public class SDWPlatformServices
{
    private static final String DOMAIN = "DefaultAuth";

    def log = null;
    
    private SDWPlatformServices(log)
    {
        this.log = log;
    }


    public void startLocalPlatformServices(String host, String port)
            throws ServiceException, RemoteException
    {
        log.debug("SDWPlatformServices.startLocalPlatformServices()");
        ServicesUtil.setLoginConfiguration(host, port);
        // XXX The next line closes BRM.log!
        ServicesUtil.startLocalServices();
        getInformationService().setIsolated();
        ServicesUtil.validateServices();
    }

    
    public static InformationServiceInterface getInformationService()
    {
        InformationServiceInterface rval = CorePlatformServices.getInformationService();
        if (rval == null) { throw new IllegalStateException("Platform services is not started"/* I18nOK:EMS */); }
        return rval;
    }
    
    
    public static void verifyPlatformServicesStarted() throws ServiceException
    {
        ServicesUtil.validateServices();
    }

 
    public static void stopPlatformServices()
    {
        try {
            ServicesUtil.terminateServices();
        } catch (ServiceException e) {
            e.printStackTrace();
        }
    }

    
    public UserContextInterface newUser(String uid, String pwd) throws RemoteException,
            ServiceException
    {
        UserContextInterface rval;
        UserServiceInterface userService = CorePlatformServices.getUserService();

        rval = userService.newUser(uid, pwd, DOMAIN);
        SessionContextInterface session = CorePlatformServices.getSessionService().newSessionContext(rval);
        rval.setSessionContext(session);
        
        return rval;
    }
}
