package com.sas.solutions.brm.discovery;

import java.util.List;

import org.apache.log4j.Logger;

import com.sas.solutions.brm.ClassIds;
import com.sas.solutions.brm.errors.BRMUserError;
import com.sas.solutions.brm.omr.GenericMetadataObject;
import com.sas.solutions.brm.omr.OMRConnection;

public class MachineDiscoverer {
    
    private static final Logger LOG = Logger.getLogger(MachineDiscoverer.class);
    OMRConnection omr = null;
    String user,pass,port,OMRHost;
    
    public MachineDiscoverer(String user, String password, String port, String hostname) {
        if(this.user==null) this.user = user;
        if(pass==null) pass =password;
        if(this.port==null) this.port=port;
        if(OMRHost==null) OMRHost=hostname;
        
        if(omr == null) {
            omr = new OMRConnection(OMRHost,port,user,pass);
            omr.connect();
        }
        
    }
    
    public String findDavServer() {
        
        return findServerByClassID(ClassIds.CLASSID_HTTP);
    }
    public String findConnectServer() {
        return findServerByClassID(ClassIds.CLASSID_CONNECT_SERVER);
    }
    public String findWorkspaceServer() {
        return findServerByClassID(ClassIds.CLSID_SAS_WORKSPACE);
    }
    public String findStoredProcessServer() {
        return findServerByClassID(ClassIds.CLSID_SASSTP_STOREDPROCESSSERVER);
    }
    public String findMySQLServer() {
        return findServerByClassID("SSS_MYSQL_TIER");
    }

    
    private String findServerByClassID(String id) {
        omr.setFoundationRepository();
        String typeName = "Machine";
        String query = "Machine[DeployedComponents/ServerComponent[@ClassIdentifier='"+id+"']]";
        List results = omr.doOMRQuery(typeName, query);
        if(results.size()==1){
            GenericMetadataObject item = (GenericMetadataObject)results.get(0);
            LOG.info("Server found for class id using ServerComponent: "+id+". It is: "+item.getName());
            return item.getName();
        }
        
        
        query = "Machine[DeployedComponents/ConfiguredComponent[@ClassIdentifier='"+id+"']]";
        results = omr.doOMRQuery(typeName, query);
        if(results.size()==1){
            GenericMetadataObject item = (GenericMetadataObject)results.get(0);
            LOG.info("Server found for class id usind ConfiguredComponent: "+id+". It is: "+item.getName());
            return item.getName();
        }
        throw new BRMUserError("BRM found an unusual server setup.  This may have to be manually configured.");
    }
    
    
    
    

}
