package com.sas.solutions.brm.logging;

import java.io.File;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.tools.ant.BuildEvent;
import org.apache.tools.ant.BuildListener;
import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;

public class BRMBuildListener implements BuildListener
{
    private String component;
    private Logger logger;
    
    public BRMBuildListener(String component, Logger logger) {
        this.component = component;
        this.logger = logger;
    }

    public Logger getLogger()
    {
        return this.logger;
    }

    public static Logger getLogger(Project project)
    {
        Vector listeners = project.getBuildListeners();
        Iterator listenerIt = listeners.iterator();
        while (listenerIt.hasNext())
        {
            // Note: Have to use the Groovy "def" keyword here instead of
            // Java-style static type declaration, or I get a really weird
            // class cast exception later.
            def b = listenerIt.next();
            // Note: instanceof operator doesn't work here because code run
            // indirectly via Ant has a different class loader. So I have to do
            // a string compare on the class name.
            if (b.getClass().getName().equals(
                "com.sas.solutions.brm.logging.BRMBuildListener"))
            {
                return b.getLogger();
            }
        }
        return null;
    }

    public void buildStarted(BuildEvent event) {
        this.logger.info("Build started for component:"+ component );

    }

    public void buildFinished(BuildEvent event) {
        this.logger.info("Build finished for component:"+ component );

    }

    public void targetStarted(BuildEvent event) {
        this.logger.info(constructMessageFront(event)+" target started");

    }

    public void targetFinished(BuildEvent event) {
        this.logger.info(constructMessageFront(event)+" target finished");

    }

    public void taskStarted(BuildEvent event) {
        this.logger.debug(constructMessageFront(event)+" task started");

    }

    public void taskFinished(BuildEvent event) {
        this.logger.debug(constructMessageFront(event)+" task finished");

    }

    public void messageLogged(BuildEvent event)
    {
        Level level = logLevelFromPriorityInt(event.getPriority());
        StringBuffer sb = new StringBuffer();
        sb.append(eventLocationPrefix(event));
        sb.append(event.getMessage());
        Throwable t = event.getException();
        if (t != null)
            this.logger.log(level, sb.toString(), t);
        else
            this.logger.log(level, sb.toString());
    }

    private Level logLevelFromPriorityInt(int priority)
    {
        switch (priority)
        {
            case Project.MSG_ERR:
                return Level.ERROR;
            case Project.MSG_WARN:
                return Level.WARN;
            case Project.MSG_INFO:
                return Level.INFO;
            case Project.MSG_DEBUG:
            case Project.MSG_VERBOSE:
                return Level.DEBUG;
            default:
                // In case some new message priority is invented later...
                return Level.WARN;
        }
    }

    private String eventLocationPrefix(BuildEvent event)
    {
        StringBuffer sb = new StringBuffer();
        Task task = event.getTask();
        if (task != null)
        {
            Location location = task.getLocation();
            if (!Location.UNKNOWN_LOCATION.equals(location))
            {
                File file = new File(location.getFileName());
                int lineNumber = location.getLineNumber();
                sb.append(file.getName());
                sb.append((char)':');
                sb.append(lineNumber);
                sb.append((char)' ');
            }
        }
        return sb.toString();
    }
    
    private String constructMessageFront(BuildEvent event)
    {
        StringBuffer sb = new StringBuffer(50);
        sb.append(eventLocationPrefix(event));
        if (event.getTask() != null)
        {
            sb.append("{"+event.getTarget()+"}["+
                    event.getTask().getTaskName()+"]");
        }
        else if (event.getTarget() != null)
        {
            sb.append("{"+event.getTarget()+"}");
        }
        return sb.toString();
    }

}
