/*
 * Copyright (c) 2010 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.logging;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.Priority;
import org.apache.log4j.spi.LoggingEvent;

/**
  * Logger that logs to console via println, using a standard format,
  * ignoring Log4j config files and the logger hierarchy.
  */
public class ConsoleLogger extends Logger
{
    private Level consoleLogLevel = Level.INFO;

    public ConsoleLogger()
    {
        super("");

        // Override protected repository field so that
        // respository.isDisabled() always returns false
        this.repository = new BRMLoggerRepository();
    }

    /**
     * Override setLevel(logLevel).
     */
    public void setLevel(Level logLevel)
    {
        this.consoleLogLevel = logLevel;
    }

    /**
     * Override getEffectiveLevel().
     */
    public Level getEffectiveLevel()
    {
        return this.consoleLogLevel;
    }

    /**
     * Override isEnabledFor(logLevel).
     */
    public boolean isEnabledFor(Level logLevel)
    {
        return (logLevel.isGreaterOrEqual(this.consoleLogLevel));
    }

    public void trace(Object msg)
    {
        log(Level.TRACE, msg);
    }

    public void trace(Object msg, Throwable e)
    {
        log(Level.TRACE, msg, e);
    }

    public void debug(Object msg)
    {
        log(Level.DEBUG, msg);
    }

    public void debug(Object msg, Throwable e)
    {
        log(Level.DEBUG, msg, e);
    }

    public void info(Object msg)
    {
        log(Level.INFO, msg);
    }

    public void info(Object msg, Throwable e)
    {
        log(Level.INFO, msg, e);
    }

    public void warn(Object msg)
    {
        log(Level.WARN, msg);
    }

    public void warn(Object msg, Throwable e)
    {
        log(Level.WARN, msg, e);
    }

    public void error(Object msg)
    {
        log(Level.ERROR, msg);
    }

    public void error(Object msg, Throwable e)
    {
        log(Level.ERROR, msg, e);
    }

    public void fatal(Object msg)
    {
        log(Level.FATAL, msg);
    }

    public void fatal(Object msg, Throwable e)
    {
        log(Level.FATAL, msg, e);
    }

    public void log(Priority priority, Object message)
    {
        log(priority, message, null);
    }

    public void log(Priority priority, Object message, Throwable t)
    {
        if (!isEnabledFor(priority))
            return;
        println("${priority}: ${message}");
        if (t != null)
            t.printStackTrace();
    }

    /**
     * Override Category.callAppenders().
     * If this method is called, something went wrong with overriding the
     * base class.
     */
    public void callAppenders(LoggingEvent event)
    {
        assert false;
    }
}
