/*
 * Created on Feb 16, 2007
 * Copyright (c) 2007 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.logging;

import java.util.Enumeration;

import org.apache.log4j.Appender;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.LogManager;
import org.apache.log4j.spi.LoggingEvent;

// DO NOT USE BRMUtil here
// It will cause Logger.getLogger() to get called prematurely

public class LogUtil
{
    /**
      * Disabled constructor - static utility class.
      */
    private LogUtil()
    {
    }

    /**
      * Get a Logger associated with logName. If loggerName is null, return
      * the root logger.
      */
    public static Logger getLogger(String loggerName)
    {
        Logger logger = null;
        if (loggerName == null)
        {
            logger = Logger.getRootLogger();
        }
        else
        {
            logger = Logger.getLogger(loggerName);
        }
        return logger;
    }

    /**
      * @return The name of the Logger that generated the event, or null if
      * there is an error trying to get the logger name.
      */
    public static String getLoggerName(LoggingEvent event)
    {
        return getLoggerName(event, false);
    }

    public static String getLoggerName(LoggingEvent event, 
            boolean reportErrors)
    {
        try
        {
            return event.getLoggerName();
        }
        catch (NoSuchMethodError e)
        {
            if (reportErrors)
            {
                System.err.println(
                    "*** Error getting logger name via event.getLoggerName()");
                throw e;
            }
        }
        return null;
    }

    public static void dumpEventInfo(LoggingEvent event)
    {
        System.out.println(event);
        System.out.println("  Event thread: " + event.getThreadName());
        System.out.println("  Current thread: " + Thread.currentThread());
        System.out.println("  Event classloader: " +
                event.getClass().getClassLoader());
        System.out.println("  Thread classloader: " +
                Thread.currentThread().getClass().getClassLoader());
        System.out.println("  System classloader: " +
                ClassLoader.getSystemClassLoader());
    }

    public static void dumpLoggers()
    {
        System.out.println("Logger info:");
        System.out.println("  Root logger:");
        dumpLogger(Logger.getRootLogger(), 4);
        System.out.println("  Current loggers:");
        for (Enumeration loggerEnum=LogManager.getCurrentLoggers();
                loggerEnum.hasMoreElements(); )
        {
            Logger logger = (Logger)loggerEnum.nextElement();
            dumpLogger(logger, 4);
        }
    }

    public static void dumpLogger(Logger logger, int indent)
    {
        String indentStr = getIndentStr(indent);
        System.out.println(indentStr + "Logger: " + logger.getName());
        System.out.println(indentStr + "  Level: " + logger.getLevel());
        System.out.println(indentStr + "  Appenders:");
        for (Enumeration appenderEnum=logger.getAllAppenders();
                appenderEnum.hasMoreElements(); )
        {
            Appender appender = (Appender)appenderEnum.nextElement();
            dumpAppender(appender, indent + 4);
        }
    }

    public static void dumpAppender(Appender appender, int indent)
    {
        String indentStr = getIndentStr(indent);
        System.out.println(indentStr + "Appender: " + appender.getName());
        System.out.println(indentStr + "  Class: " + 
                                appender.getClass().getName());
    }

    // Return a string containing the number of spaces given in the indent
    // paramater
    public static String getIndentStr(int indent)
    {
        StringBuffer sb = new StringBuffer();
        for (int i=0; i<indent; ++i)
            sb.append(" ");
        return sb.toString();
    }

    /**
      * @return The URL used to locate the Log4j configuration file.
      */
    public static java.net.URL getLog4jConfigURL()
    {
        String resource = System.getProperty("log4j.configuration");
        if (resource == null)
            resource = "log4j.properties";
        java.net.URL url = null;
        try
        {
            url = new java.net.URL(resource);
        }
        catch (java.net.MalformedURLException e)
        {
            url = org.apache.log4j.helpers.Loader.getResource(resource);
        }
        return url;
    }

    public static List getLogFiles()
    {
        // This code is problematic because it doesn't indicate the
        // corrrect log file for the brm agent.
        List logFiles = new ArrayList();
        Logger root = Logger.getRootLogger();
        Enumeration appenders = root.getAllAppenders();
        while (appenders.hasMoreElements())
        {
            Appender appender = (Appender)appenders.nextElement();
            if (appender instanceof FileAppender)
            {
                logFiles.add(((FileAppender)appender).getFile());
            }
        }
        return logFiles;
    }
}
