/*
 * Created on Fri Oct. 15, 2010
 * Copyright (c) 2010 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.logging;

import java.io.ObjectOutputStream;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.spi.LoggingEvent;

/**
 * Wrapper of Log4j Logger that forwards all log events over an
 * ObjectOutputStream. All events are forwarded; filtering based on log
 * level is done by the receiver.
 */
public class ObjectOutputStreamLogger extends Logger
{
    private ObjectOutputStream objectOutputStream = null;

    public ObjectOutputStreamLogger(String name,
                                    ObjectOutputStream objectOutputStream)
    {
        super(name); // This will be the Logger name in LoggingEvent objects

        // Override protected repository field so that
        // respository.isDisabled() always returns false
        this.repository = new BRMLoggerRepository();

        this.objectOutputStream = objectOutputStream;
        setLevel(Level.INFO); // Default level
    }

  // Override of org.apache.log4j.Category.callAppenders
  /**
     Call the appenders in the hierrachy starting at
     <code>this</code>.  If no appenders could be found, emit a
     warning.

     <p>This method calls all the appenders inherited from the
     hierarchy circumventing any evaluation of whether to log or not
     to log the particular log request.

     @param event the event to log.  */
  public
  void callAppenders(LoggingEvent event) {
    // Do NOT call the base class
    //super.callAppenders(event);
    if (this.objectOutputStream == null)
        return;
    try
    {
        this.objectOutputStream.writeObject(event);
        this.objectOutputStream.flush();
        return;
    }
    catch (IOException e)
    {
        // Controller is shut down, can't send remote log messages
        // Stop using the output stream
        this.objectOutputStream = null;
        System.err.println("Connection to logger went down: " + e);
    }
    catch (Throwable t)
    {
        System.err.println("Error sending event to log server: " + t);
    }
  }
}

// end-of-file
