/*
 * Created on Feb 14, 2007
 * Copyright (c) 2007 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.logging;

import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.net.ServerSocket;
import java.net.Socket;

import org.apache.log4j.spi.LoggingEvent;

import com.sas.solutions.brm.logging.MonitoredLogger;

/**
  * Remote logging server.
  * This class handles TCP messages containing serialized LogEvent objects,
  * allowing log events generated in one JVM to be logged in another.
  */
public class RemoteLoggingServer 
{
    private volatile ObjectInputStream objInputStream = null;
    private MonitoredLogger logger = null;
    private Object result = null;
    private Throwable networkError = null;

    /**
     * @param socket  An open TCP socket
     * @param logger  An instance of MonitoredLogger with logEvent method
     */
    public RemoteLoggingServer(Socket socket, MonitoredLogger logger)
    {
        this.objInputStream = new ObjectInputStream(socket.getInputStream());
        this.logger = logger;
    }

    public void run()
    {
        Object obj = null;
        while (this.objInputStream != null)
        {
            this.networkError = null;
            try
            {
                obj = this.objInputStream.readObject();
            }
            catch (EOFException e)
            {
                // Normal end of transmission
                obj = null;
            }
            catch (IOException e)
            {
                // Abnormal disconnect
                this.networkError = e;
                obj = null;
            }
            if (obj == null)
                break;
            if (obj instanceof LoggingEvent)
            {
                append((LoggingEvent)obj);
                continue;
            }
            this.result = obj;
            break;
        }
        close();
    }

    public void append(LoggingEvent event)
    {
        if (this.logger == null)
            return;
        this.logger.logEvent(event);
    }

    public void close()
    {
        ObjectInputStream ois = this.objInputStream;
        this.objInputStream = null;
        if (ois != null)
        {
            ois.close();
        }
    }

    public Object getResult()
    {
        return this.result;
    }

    public Throwable getNetworkError()
    {
        return this.networkError;
    }
}
