/*
 * Created on July 7, 2011
 * Copyright (c) 2011 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.migrate;

import com.sas.solutions.brm.omr.GenericMetadataObject;
import com.sas.solutions.brm.omr.OMRConnection;

class ConfigObjDumper extends ConfigObjVisitorBase
{
    public ConfigObjDumper(omr, tempDir, logger)
    {
        super(omr, tempDir, logger);
    }

    def dumpReposPrefixMap()
    {
        def reposPrefixMap = new Properties();
        this.omr.getRepositories().each
        { repos ->
            // Note: Prefix of object ID is postfix of repository ID
            def postfix = repos.getId().split(/\./)[1];
            reposPrefixMap.setProperty(postfix, repos.getName());
        }
        def tempConfigDir = new File(this.tempDir, 'config');
        tempConfigDir.mkdirs();
        def reposPrefixMapFile = new File(tempConfigDir, 
                                          'repos-prefix-map.properties');
        this.logger.info("Dumping repository prefix map to file: ${reposPrefixMapFile}");
        def reposPrefixMapOut = new FileOutputStream(reposPrefixMapFile);
        reposPrefixMap.store(reposPrefixMapOut, ' Repository prefix map');
        reposPrefixMapOut.close();
    }

    def dump()
    {
        this.configObjsDir.mkdirs();
        def dumpCompleteFile = new File(this.configObjsDir, 'dumpcomplete.txt');
        if (dumpCompleteFile.exists())
            dumpCompleteFile.delete();
        dumpReposPrefixMap();
        def configobjtypesFile = new File(this.configObjsDir, 
                                          'configobjtypes.txt');
        configobjtypesFile.write('');
        this.configObjTypes.each
        { configObjType ->
            configobjtypesFile.append(configObjType + '\r\n');
        }
        def configobjpathsFile = new File(this.configObjsDir, 
                                          'configobjpaths.txt');
        configobjpathsFile.write('');
        def contentobjsbynameFile = new File(this.configObjsDir,
                                             'contentobjsbyname.txt');
        contentobjsbynameFile.write('');
        this.logger.info("Dumping configuration-related objects in metadata to directory ${this.configObjsDir}");

        def dumpObject = { obj ->
            this.logger.debug("Dumping ${obj.getTypeName()} Id=${obj.getId()} Name=\"${obj.getName()}\"");
            def objTypeName = obj.getTypeName();
            if ((objTypeName in this.contentBearingConfigObjTypes)||
                 hasContentAssoc(obj))
            {
                contentobjsbynameFile.append("${objTypeName}:${obj.getName()}=${obj.getId()}\n", 'UTF-8');
            }
            def objTypeDir = new File(this.configObjsDir, objTypeName);
            objTypeDir.mkdirs();
            def objFile = new File(objTypeDir, obj.getId() + '.xml');
            objFile.write(obj.getXML(), 'UTF-8');
        }

        def dumpFolder = { obj, path ->
            this.logger.debug("Recording folder Tree path: ID=${obj.getId()} Name=\"${obj.getName()}\" path=\"${path}\"");
            configobjpathsFile.append("${obj.getId()}=${path}\r\n", 'UTF-8');
        }

        def handleBIPService = { obj ->
            this.logger.info("Recording BIP Service SoftwareComponent ID: ${obj.getId()}");
            def bipserviceidsFile = new File(this.configObjsDir, 
                                            'bipserviceids.txt');
            this.logger.info("Adding ${obj.getId()} to ${bipserviceidsFile}");
            bipserviceidsFile.append(obj.getId() + '\r\n', 'UTF-8');
        }

        visitConfigObjs([
            'visitConfigObj': dumpObject,
            'visitFolder': dumpFolder,
            'visitBIPService': handleBIPService,
            ]);

        dumpCompleteFile.write("${this.numVisitedObjs}\r\n");
        this.logger.info("Dumped ${this.numVisitedObjs} total configuration-related objects");
    }
}
