/*
 * Created on Nov. 11, 2010
 * Copyright (c) 2010 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.migrate;

import com.sas.solutions.brm.omr.OMRConnection;

class HostDumper extends HostFinder
{
    def out = System.out;

    public HostDumper(OMRConnection omr, MigrateConfig components,
                        Properties brmprops, String filename)
    {
        super(omr, components, brmprops);
        if (filename != null)
        {
            def autoFlush = true;
            this.out = new PrintWriter(new FileWriter(filename), autoFlush);
        }
    }

    def visitMachines()
    {
        out.println("********");
        out.println("Machines");
        out.println("********");
        super.visitMachines();
        out.println();
    }

    def handleMachine(machine)
    {
        super.handleMachine(machine);
        def name = machine.getName();
        def components = machine.getAssociation('DeployedComponents');
        out.println();
        out.println("Machine: (${machine.getId()}) ${name}");
        out.println("DeployedComponents:");
        components.each
        { component ->
            out.println("    ${component.getTypeName()}: (${component.getId()}) \"${component.getName()}\"");
        }
        def trees = machine.getAssociation('Trees');
        if (trees == null || trees.size() == 0)
            return;
        out.println("Trees:");
        trees.each
        { tree ->
            tree = getObject(tree);
            out.println("    (${tree.getId()}) \"${tree.getName()}\"");
            tree.getAssociation('SoftwareComponents').each
            { treecomp ->
                out.println("        ${treecomp.getTypeName()}: (${treecomp.getId()}) \"${treecomp.getName()}\"");
            }
        }
    }

    def visitDocuments()
    {
        out.println("****************************************");
        out.println("Document objects with machine references");
        out.println("****************************************");
        super.visitDocuments();
        out.println();
    }

    def newRepository(reposname)
    {
        out.println();
        out.println('====================');
        out.println(reposname);
        out.println('====================');
    }

    def handleDocument(document, objs)
    {
        super.handleDocument(document, objs);
        def uri = document.getAttribute('URI');
        out.println("(${document.getId()}) \"${document.getName()}\" URI=${uri}");
        objs.each
        { obj ->
            obj = getObject(obj);
            def classId = obj.getAttribute('ClassIdentifier');
            def classIdStr = quoteNonNullStr(classId);
            out.println("    ${obj.getTypeName()}: (${obj.getId()}) \"${obj.getName()}\" ClassIdentifier=${classIdStr}");
        }
    }

    def visitLogins()
    {
        out.println("******************************************");
        out.println("Login objects containing DOMAIN\\ prefixes");
        out.println("******************************************");
        super.visitLogins();
        out.println();
    }

    def handleLogin(login)
    {
        super.handleLogin(login);
        def nameStr = quoteNonNullStr(login.getName());
        def userIDStr = quoteNonNullStr(login.getAttribute('UserID'));
        out.println("(${login.getId()}) Name=${nameStr} UserID=${userIDStr}");
    }

    def visitProperties()
    {
        out.println("**************************************");
        out.println("Property objects containing host names");
        out.println("**************************************");
        super.visitProperties();
        out.println();
    }

    def handleProperty(property)
    {
        def name = property.getName();
        if (name == null)
            return;
        def defaultValue = property.getAttribute('DefaultValue');
        if (defaultValue == null)
            return;
        super.handleProperty(property);
        out.println("(${property.getId()}) \"${name}\" DefaultValue=\"${defaultValue}\"");
        if (!name.endsWith('.xmlKey.txt'))
            return;
        def obj = property.getAssociatedObject('AssociatedObject');
        if (obj != null && obj.getTypeName() == 'SASClientConnection')
        {
            obj = getObject(obj);
            out.println("    Associated ${obj.getTypeName()}: (${obj.getId()}) \"${obj.getName()}\"");
            def source = obj.getAssociatedObject('Source');
            if (source != null)
            {
                source = getObject(source);
                def classId = source.getAttribute('ClassIdentifier');
                def classIdStr = quoteNonNullStr(classId);
                out.println("        Source ${source.getTypeName()}: (${source.getId()}) \"${source.getName()}\" ClassIdentifier=${classIdStr}");
            }
            return;
        }
    }

    def visitSASClientConnections()
    {
        out.println("*************************************************");
        out.println("SASClientConnection objects containing host names");
        out.println("*************************************************");
        super.visitSASClientConnections();
        out.println();
    }

    def handleSASClientConnection(connection)
    {
        super.handleSASClientConnection(connection);
        def name = connection.getName();
        def nameStr = quoteNonNullStr(name);
        def remoteAddress = connection.getAttribute('RemoteAddress');
        def remoteAddressStr = '';
        if (remoteAddress != null)
            remoteAddressStr = " RemoteAddress=\"${remoteAddress}\"";
        def serverShortName = connection.getAttribute('ServerShortName');
        def serverShortNameStr = '';
        if (serverShortName != null)
            serverShortNameStr = " ServerShortName=\"${serverShortName}\"";
        out.println("(${connection.getId()}) ${nameStr}${remoteAddressStr}${serverShortNameStr}");
    }

    def visitServerComponents()
    {
        out.println("*********************************************");
        out.println("ServerComponent objects containing host names");
        out.println("*********************************************");
        super.visitServerComponents();
        out.println();
    }

    def handleServerComponent(component)
    {
        super.handleServerComponent(component);
        def name = component.getName();
        def nameStr = quoteNonNullStr(name);
        out.println("(${component.getId()}) ${nameStr}");
    }

    def visitSoftwareComponents()
    {
        out.println("***********************************************");
        out.println("SoftwareComponent objects containing host names");
        out.println("***********************************************");
        super.visitSoftwareComponents();
        out.println();
    }

    def handleSoftwareComponent(component)
    {
        super.handleSoftwareComponent();
        def name = component.getName();
        def nameStr = quoteNonNullStr(name);
        out.println("(${component.getId()}) ${nameStr}");
    }

    def visitTCPIPConnections()
    {
        out.println("*********************************************");
        out.println("TCPIPConnection objects containing host names");
        out.println("*********************************************");
        super.visitTCPIPConnections();
        out.println();
    }

    def handleTCPIPConnection(connection)
    {
        super.handleTCPIPConnection(connection);
        def name = connection.getName();
        def nameStr = quoteNonNullStr(name);
        def hostName = connection.getAttribute('HostName');
        def service = connection.getAttribute('Service');
        def serviceStr = '';
        if (service != null)
            serviceStr = " Service=\"${service}\"";
        out.println("(${connection.getId()}) ${nameStr} HostName=\"${hostName}\"${serviceStr}");
        def classId = null;
        def source = connection.getAssociatedObject('Source');
        if (source != null)
        {
            source = getObject(source);
            classId = source.getAttribute('ClassIdentifier');
            def classIdStr = quoteNonNullStr(classId);
            out.println("    Source ${source.getTypeName()}: (${source.getId()}) \"${source.getName()}\" ClassIdentifier=${classIdStr}");
        }
    }

    def visitTrees()
    {
        out.println("**********************************************");
        out.println("Tree objects potentially containing host names");
        out.println("**********************************************");
        super.visitTrees();
        out.println();
    }

    def handleTree(tree)
    {
        super.handleTree(tree);
        def name = tree.getName();
        def nameStr = quoteNonNullStr(name);
        out.println("(${tree.getId()}) ${nameStr}");
    }

    def visitTextStoreXMLObjs()
    {
        out.println("************************************************");
        out.println("TextStore objects with XML containing host names");
        out.println("************************************************");
        super.visitTextStoreXMLObjs();
        out.println();
    }

    def handleXMLValue(obj, parent, node, value)
    {
        if (this.numHostsInThisText == 1)
        {
            out.println();
            out.println("(${obj.getId()}) ${obj.getName()}: StoredText (XML):");
        }
        def parentName = '';
        def grandParentName = '';
        if (parent != null)
        {
            parentName = parent.getNodeName();
            def grandParent = parent.getParentNode();
            if (grandParent != null)
                grandParentName = grandParent.getNodeName();
        }
        out.println("${grandParentName}/${parentName}/${node}");
        def host = null;
        if (grandParentName == 'InformationService' && parentName == 'ReposDef')
        {
            if (node.getNodeName() != 'host')
                return;
            host = value;
            def protocol = parent.getAttribute('protocol');
            out.println("    parsed host: \"${host}\"");
            out.println("       protocol: \"${protocol}\"");
            return;
        }
        if (grandParentName == 'UserService' && parentName == 'Profile')
        {
            if (node.getNodeName() != 'url')
                return;
            def u = null;
            try
            {
                u = new java.net.URI(value);
            }
            catch (java.net.URISyntaxException e)
            {
                // Can't replace the host and port if I can't parse it
                return;
            }
            host = u.getHost();
            def port = u.getPort();
            out.println("    parsed host: \"${host}\"");
            out.println("           port: \"${port}\"");
            return;
        }
    }

    def visitTextStorePropertiesObjs()
    {
        out.println("*******************************************************");
        out.println("TextStore objects with Properties containing host names");
        out.println("*******************************************************");
        super.visitTextStorePropertiesObjs();
        out.println();
    }

    // This is only used for printing property lines with host names in them.
    // TextStore Properties objects are fixed during the BRM restore phase.
    def handlePropertiesTextLine(obj, line)
    {
        if (this.numHostsInThisText == 1)
        {
            out.println();
            out.println("(${obj.getId()}) \"${obj.getName()}\" StoredText (Properties):");
        }
        out.println(line);
    }

    def visitPromptGroups()
    {
        out.println("**************************************************");
        out.println("PromptGroup objects with XML containing host names");
        out.println("**************************************************");
        super.visitPromptGroups();
        out.println();
    }

    def handlePromptGroupXMLValue(obj, parent, node, value)
    {
        if (this.numHostsInThisText == 1)
        {
            out.println();
            out.println("(${obj.getId()}) \"${obj.getName()}\" GroupInfo (XML):");
        }
        out.println(node);
    }

    def visitHosts()
    {
        super.visitHosts();
        out.println();
        out.println("${sprintf('%5d', this.searchedObjectsCount)} objects with potential host names");
    }
}
