package com.sas.solutions.brm.migrate;

import com.sas.solutions.brm.util.BRMUtil;

public class HostObjDumper
{
    // TypeName, Query
    def dumpTable = [
        ['Property', "@Name='omr_host'"],
        ['Property', "@Name='Metadata.Host'"],
        ['Property', "@Name='Metadata.Port'"],
        ['Property', "@Name='Metadata.Repository'"],
        ['Property', "@Name='Metadata.Userid'"],
        ['Property', "@Name='bid.Midtier.trusted.URLs'"],
        ['Property', "@Name='SPMProvider.Midtier.trusted.URLs'"],
        ['Property', "@Name='soltnsmid.jdbc.connection.userid'"],

        ['SASClientConnection', "*[Source/ServerComponent[@ClassIdentifier='0217E202-B560-11DB-AD91-001083FF6836']]"],
        ['SASClientConnection', "*[Source/ServerComponent[@ClassIdentifier='E803E04B-AAF0-4C74-86A6-F7008AA702DC']]"],
        ['SASClientConnection', "*[Source/ServerComponent[@ClassIdentifier='028e4060-d545-11d5-880d-aa0004006d06']]"],
        ['SASClientConnection', "*[Source/ServerComponent[@ClassIdentifier='SHARE']]"],

        ['ServerComponent', "*[@ClassIdentifier='IOM Spawner']"],
        ['ServerComponent', "*[@ClassIdentifier='SHARE']"],

        ['SoftwareComponent', "*[@ClassIdentifier='mysqldbs']"],
        ['SoftwareComponent', "*[@ClassIdentifier='webappsrv']"],

        ['Tree', "*[Members/SoftwareComponent[@ClassIdentifier='mysqldbs']]"],
    ];

    def omr = null;
    def tempDir = null;
    def logger = null;

    public HostObjDumper(omr, tempDir, logger)
    {
        this.omr = omr;
        this.tempDir = tempDir;
        this.logger = logger;
    }

    def dumpObjectsByQueryTable(objectsDir)
    {
        this.logger.info("Dumping metadata objects that may contain host names to directory: ${objectsDir}");
        def objectsFile = new File(objectsDir, 'objects.txt');
        def autoFlush = true;
        def out = new PrintWriter(
                    new OutputStreamWriter(
                        new FileOutputStream(objectsFile), 
                        'UTF-8'), 
                    autoFlush);
        try
        {
            this.dumpTable.each
            { row ->
                def typename = row[0];
                def query = row[1];
                def objs = this.omr.doOMRQuery(typename, query, true);
                this.logger.info("Query: ${typename} \"${query}\": returned ${objs.size()} object(s)");
                if (objs.size() == 0)
                {
                    // Sometimes the Tree object does not exist, that's Ok
                    return;
                }
                if (objs.size() > 1)
                {
                    this.logger.error("Metadata query: ${typename} \"${query}\": multiple objects match query, not dumping any of them.");
                    return;
                }
                def obj = objs[0];
                out.println("${obj.getId()},${obj.getTypeName()},${query}");
                dumpObject(obj, objectsDir);
            }
        }
        finally
        {
            out.close();
        }
    }

    def dumpObject(obj, componentsDir)
    {
        def objFile = new File(componentsDir, obj.getId() + '.xml');
        BRMUtil.writeStringToFile(objFile, obj.getXML());
    }

    def dump()
    {
        def objectsDir = new File(this.tempDir, 'objects-with-hostnames');
        objectsDir.mkdirs();
        this.omr.setFoundationRepository();
        dumpObjectsByQueryTable(objectsDir);
    }
}
