package com.sas.solutions.brm.migrate;

import com.sas.solutions.brm.omr.GenericMetadataObject;
import com.sas.solutions.brm.omr.ObjectInfoHandler;
import com.sas.solutions.brm.omr.OMRUtil;

public class HostObjLoader
{
    def omr = null;
    def tempDir = null;
    def logger = null;
    def reposPrefixMap = [:];

    public HostObjLoader(omr, tempDir, logger)
    {
        this.omr = omr;
        this.tempDir = tempDir;
        this.logger = logger;
    }

    def loadObjectsFromDirectory(objectsDir)
    {
        this.logger.info("Re-loading objects that may contain host names from directory: ${objectsDir}");
        if (!objectsDir.isDirectory())
        {
            this.logger.warn("Directory \"${objectsDir}\" does not exist, can not load objects.");
            return;
        }
        def objectsFile = new File(objectsDir, 'objects.txt');
        this.logger.info("Reading object info from file: ${objectsFile}");
        def reader = new BufferedReader(
                        new InputStreamReader(
                            new FileInputStream(objectsFile), 
                            'UTF-8'));
        try
        {
            def linenum = 0;
            while (true)
            {
                def line = reader.readLine();
                if (line == null)
                    break;
                linenum += 1;
                def parts = line.split(',', 3);
                if (parts.length != 3)
                {
                    this.logger.error("File ${objectsFile}: line ${linenum}: format error: expected 3 fields, found ${parts.length}");
                    return;
                }
                def objid = parts[0];
                def typename = parts[1];
                def query = parts[2];
                def filename = objid + '.xml';
                def objFile = new File(objectsDir, filename);
                def srcObj = OMRUtil.loadObjectFromFile(objFile);
                if (srcObj == null)
                {
                    this.logger.error("File ${objectsFile}: line ${linenum}: could not load object from file: ${objFile}");
                    continue;
                }
                this.logger.info("Loaded source object: ${srcObj.getTypeName()}: (${srcObj.getId()}) \"${srcObj.getName()}\"");
                def objs = this.omr.doOMRQuery(typename, query, true);
                this.logger.info("Query: ${typename} \"${query}\": returned ${objs.size()} object(s)");
                if (objs.size() == 0)
                {
                    // Sometimes the Tree object does not exist, that's Ok
                    continue;
                }
                if (objs.size() > 1)
                {
                    this.logger.error("Metadata query: ${typename} \"${query}\": multiple objects match query, can't match with loaded object.");
                    continue;
                }
                def dstObj = objs[0];
                this.logger.info("Updating destination object: ${dstObj.getTypeName()} (${dstObj.getId()}) \"${dstObj.getName()}\"");
                OMRUtil.copyObjectAttributes(srcObj, dstObj);
                this.omr.store(dstObj);
            }
        }
        finally
        {
            reader.close();
        }
    }

    def load()
    {
        def objectsDir = new File(this.tempDir, 'objects-with-hostnames');
        omr.setFoundationRepository();
        loadObjectsFromDirectory(objectsDir);
    }
}
