/*
 * Created on Jun 8, 2006
 * Copyright (c) 2006 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.migrate;

import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import com.sas.solutions.brm.Constants;
import com.sas.solutions.brm.errors.BRMConfigurationException;

/**
 * Migration Configuration - tells the migration process what need to be
 * changed and what the new values should be, based on a configuration file
 * in INI format.
 */
public class MigrateConfig
{
    public File iniFile = null;
    private Map sectionMap = null;
    
    public MigrateConfig() throws BRMConfigurationException
    {
        java.net.URL url =
            ClassLoader.getSystemResource(Constants.MIGRATE_CONFIG_FILE);
        if (url == null)
        {
            throw new BRMConfigurationException(
                    "Could not find config file on classpath: " +
                    Constants.MIGRATE_CONFIG_FILE);
        }
        File inputFile = new File(url.getFile());
        try
        {
            load(inputFile);
        }
        catch (IOException e)
        {
            throw new BRMConfigurationException(
                    "Could not load migration config file: " + inputFile, e);
        }
    }

    public MigrateConfig(File inputFile) throws BRMConfigurationException
    {
        try
        {
            load(inputFile);
        }
        catch (IOException e)
        {
            throw new BRMConfigurationException(
                    "Could not load migration config file: " + inputFile, e);
        }
    }
    
    public void load(File inputFile) 
        throws FileNotFoundException, IOException
    {
        this.iniFile = inputFile;
        this.sectionMap = new HashMap();
        BufferedReader br = new BufferedReader(new FileReader(inputFile));
        String line = "";
        String sectionName = "";
        while ((line=br.readLine()) != null)
        {
            line = line.trim();
            if (line.length() == 0)
            {
                continue;
            }
            else if (line.startsWith(";"))
            {
                continue;
            }
            else if (line.startsWith("[") && line.endsWith("]"))
            {
                sectionName = line.substring(1, line.length()-1);
                sectionName = sectionName.trim();
            }
            else
            {
                int eqIndex = line.indexOf("=");
                if (eqIndex >= 0)
                {
                    String key = line.substring(0, eqIndex);
                    key = key.trim();
                    String value = line.substring(eqIndex + 1);
                    value = value.trim();
                    Map section = (Map)this.sectionMap.get(sectionName);
                    if (section == null)
                    {
                        section = new HashMap();
                        this.sectionMap.put(sectionName, section);
                    }
                    section.put(key, value);
                }
            }
        }
    }

    public String getValue(String sectionName, String valueName)
    {
        Map section = (Map)this.sectionMap.get(sectionName);
        if (section == null)
        {
            return null;
        }
        return section.get(valueName);
    }

    /**
      * Get all of the key-value pairs in a section of the
      * migrate.ini file.
      */
    public Map getSection(String sectionName)
    {
        return this.sectionMap.get(sectionName);
    }
}
