/*
 * Created on July 1, 2010
 * Copyright (c) 2010 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.migrate;

class PathDumper extends PathVisitorBase
{
    def out = System.out;

    def numPathsInThisObj = 0;

    def PathDumper(omr, filename)
    {
        super(omr);
        if (filename != null)
        {
            def autoFlush = true;
            this.out = new PrintWriter(new FileWriter(filename), autoFlush);
        }
    }
 
    /////////////////////////////////////////////////////////////////////
    // Begin overridable methods

    def handleStart()
    {
        out.println("Dumping file and directory paths in metadata on server ${omr.getHost()}");
    }

    def handleRepository(repos)
    {
        def reposname = repos.getName();
        out.println();
        out.println('========================');
        out.println("Repository: ${reposname}");
        out.println('========================');
    }

    def handleType(typename)
    {
        out.println();
        out.println('----------------');
        out.println("Type: ${typename}");
        out.println('----------------');
    }

    def handleAttrVal(obj, name, attrname, attrval)
    {
        if (obj.getTypeName() == 'TextStore')
            out.println(); // Consistent formatting for TextStore objects
        out.println("(${obj.getId()}) ${name}: ${attrname}=${attrval}");
    }

    def handleXML(obj, text)
    {
        this.numPathsInThisObj = 0;
        return super.handleXML(obj, text);
    }

    def handleXMLValue(obj, parent, node, value)
    {
        this.numPathsInThisObj += 1;
        if (this.numPathsInThisObj == 1)
        {
            out.println();
            out.println("(${obj.getId()}) ${obj.getName()}: StoredText: <XML>");
        }
        def parentName = '';
        def grandParentName = '';
        if (parent != null)
        {
            parentName = parent.getNodeName();
            def grandParent = parent.getParentNode();
            if (grandParent != null)
                grandParentName = grandParent.getNodeName();
        }
        out.println("Path-like XML: ${grandParentName}/${parentName}/${node}");
    }

    def handleMultiLineText(obj, text)
    {
        this.numPathsInThisObj = 0;
        return super.handleMultiLineText(obj, text);
    }

    def handleTextLine(obj, line)
    {
        this.numPathsInThisObj += 1;
        if (this.numPathsInThisObj == 1)
        {
            out.println();
            out.println("(${obj.getId()}) ${obj.getName()}: StoredText: <multi-line text>");
        }
        out.println(line);
        return line;
    }

    // End overridable methods
    /////////////////////////////////////////////////////////////////////

    def close()
    {
        if (this.out != System.out)
        {
            this.out.close();
        }
    }
}
