/*
 * Created on July 1, 2010
 * Copyright (c) 2010 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.migrate;

import org.w3c.dom.Node;

import com.sas.solutions.brm.omr.GenericMetadataObject;

class PathVisitorBase extends OMRVisitorBase
{
    def searchedObjectsCount = 0;
    def objectsWithPathCount = 0;
    def objectHadPath = false;

    public PathVisitorBase(omr)
    {
        super(omr);
    }

    def visitPaths()
    {
        def types = [
            ['Directory', ['Name', 'DirectoryName']],
            ['File', ['FileName']],
            ['Property', ['DefaultValue']],
            ['TextStore', ['StoredText']],
            ];
        handleStart();
        omr.getRepositories().each
        { repos ->
            omr.setRepository(repos.getName());
            handleRepository(repos);
            types.each
            { typerecord ->
                def typename = typerecord[0];
                def attrnames = typerecord[-1];
                handleType(typename);
                omr.doOMRQuery(typename, '', true).each
                { obj ->
                    this.searchedObjectsCount += 1;
                    handleObj(obj, attrnames);
                }
            }
        }
    }
 
    def isPathLike(GenericMetadataObject obj, String attrname)
    {
        def typename = obj.getTypeName();
        def name = obj.getName();
        def s = obj.getAttribute(attrname);
        // Generic handling
        if (s == null)
            return false;
        if (!hasSlash(s))
            return false;
        if (s.startsWith('http:'))
            return false;
        if (s =~ /^[A-Za-z.]*[+]?omi:\/\//)
            return false;
        if (attrname == 'Desc' || attrname == 'ChangeState' || 
                attrname == 'Expression' || attrname == 'TextType' || 
                attrname == 'PasswordHash' ||
                attrname == 'UserID' || attrname == 'SetRole' ||
                attrname == 'AuthenticationType')
            return false;
        if ((attrname.toUpperCase() in ['URL', 'URI']) && 
                (!s.startsWith('file:')))
            return false;
        // Type-specific handling
        if (typename == 'Directory')
        {
            // Exclude DAV URL
            def component = obj.getAssociatedObject('DeployedComponents');
            if (component != null)
            {
                if (component.getName().startsWith('SAS Content Server'))
                    return false;
            }
            if (s.startsWith('!SASROOT'))
                return false;
            return true;
        }
        if (typename == 'Property')
        {
            def lcname = name.toLowerCase();
            if ((lcname in ['url', 'uri']) && (!s.startsWith('file:')))
                return false;
            if (
                name.endsWith('USER_ID') || lcname.endsWith('user') ||
                name.endsWith('ADMIN') || name.endsWith('TRST') ||
                lcname.endsWith('userid') || name.endsWith('DAV_BASE')||
                name == 'Model' ||
                name == 'Namespace' ||
                name == 'BaseNamespace' ||
                name == 'VMS Common Arguments' || 
                name == 'ProfileName' ||
                name == 'Interface Model' ||
                name == 'Path to Users Root Folder' ||
                name.startsWith('Server.RTDM') ||
                name == 'Viewer' ||
                name == 'OLAPCubeConnection' ||
                name == 'DeploymentTargets' ||
                name == 'App.ProfileKey' ||
                name == 'WebApp.ExcludedErrorAttributesList' ||
                name == 'WebApp.WebService' ||
                name == 'IMapExport.IMapTableLoc' ||
                name == 'Update Source Table (YES/NO)' ||
                name == 'ETL Start Date/Time' ||
                name == 'ETL End Date/Time' ||
                name == 'Prompt for Userid/Password' ||
                name == 'Proc Upload/Download Options' ||
                name == 'DiagramXML' ||
                name == 'informationMapURL' ||
                name == 'DataExplorationURL'
                )
                return false;
            if (name.startsWith('bid.'))
            {
                if (name.endsWith('.root') ||
                        name.endsWith('.metadataRootFolder') ||
                        name.endsWith('SystemLocation') ||
                        (name =~ /Root[^.]*Location$/))
                    return false;
            }
            if (s.startsWith('!SASROOT'))
                return false;
            // The Property names of the PromptDefinitionReference XML
            // properties don't follow any pattern, so have to scan value.
            if (s.startsWith('<?xml') &&
                    s.indexOf('PromptDefinitionReference') >= 0)
                return false;
            // Names of SBIP Property objects don't follow a pattern
            // So have to scan property value
            if (s.startsWith('SBIP:'))
                return false;
        }
        if (typename == 'TextStore')
        {
            if (name in [
                'Setinit text', // We don't touch the SETINIT text
                'SAS 9.1.3 to 9.2 Folder Remapping',
                ])
            {
                return false;
            }
            else if (isSAS(obj, s))
            {
                // SAS Code -- we don't touch it, so return false
                return false;
            }
            else if (s.startsWith('00000007<?xml') &&
                (s.indexOf('<InformationMap') >= 0))
            {
                // OLAP Information Map
                return false;
            }
            else
            {
                // Some other kind of text
                return true;
            }
        }
        if (typename == 'AttributeProperty' && attrname == 'DefaultValue' && (
                name == 'PropertyTabs.Application.Service.Name.txt' ||
                name == 'Server.Http.Property.BasePath.Name.xmlKey.txt' ||
                name == 
                   'Connection.Property.AuthenticationType.Name.xmlKey.txt' ||
                name ==
                   'server.SASSPDS.AttProp.Authtype.name.SASServer.xmlKey.txt'||
                name == 'Server.Common.Property.ProductName.Name.xmlKey.txt' ||
                name == 'Server.Common.Property.Vendor.Name.xmlKey.txt' ||
                name == 'Authentication Type'))
            return false;
        if (typename == 'Extension' && name == 'URI')
            return false;
        if (typename == 'TCPIPConnection' && name.endsWith('URI'))
            return false;
        if (typename == 'PropertySet' && name.startsWith('Profile/'))
            return false;
        if (typename == 'ServerComponent' && attrname == 'ProductName')
            return false;
        if (typename == 'PropertyType' && attrname == 'Name')
            return false;
        if (typename == 'ClassifierMap' && attrname == 'Name')
            return false;
        if (typename == 'TransformationStep' && attrname == 'Name')
            return false;
        if (typename == 'TypeDefinition' && attrname == 'DefaultLocation')
            return false;
        if (typename == 'Prompt' && attrname == 'PromptInfo')
            return false; // contains XML
        if (typename == 'PromptGroup' && attrname == 'GroupInfo')
            return false; // contains XML
        if (typename == 'InternalLogin' && attrname == 'Salt')
            return false;
        return true;
    }

    def static hasSlash(s)
    {
        return (s.indexOf('/') >=0 || s.indexOf('\\') >= 0);
    }

    // Return true if the XML node should be treated as potentially
    // containing a file or directory path of interest, false otherwise.
    def nodeHasPath(parent, node, value)
    {
        if (!hasSlash(value))
            return false;
        if (value in [
                'user/password', 
                'SAS/CONNECT', 
                'Prompt for Userid/Password',
                'User/Password',
                'New SAS/CONNECT Server Wizard',
                ])
            return false;
        if (value =~ /^[A-Za-z.]*[+]?omi:\/\//)
            return false;
        def parentName = '';
        def grandParentName = '';
        if (parent != null)
        {
            parentName = parent.getNodeName();
            def grandParent = parent.getParentNode();
            if (grandParent != null)
                grandParentName = grandParent.getNodeName();
        }
        if (node.getNodeType() == Node.ATTRIBUTE_NODE)
        {
            def attrname = node.getNodeName();
            if (parentName == 'InformationMap' && 
                    attrname == 'absolutePathLocation')
                return false;
            if (attrname in [
                    'AuthenticationType',
                    'ContentType',
                    'Namespace',
                    ])
                return false;
            if ((attrname.toUpperCase() in ['URL', 'URI']) &&
                    (!value.startsWith('file:')))
                return false;
            if (attrname.startsWith('xmlns:'))
                return false;
            // XMLSELECT-like matching on various object types
            if ((attrname in ['match', 'filter']) && value.startsWith('*'))
                return false;
            if (parentName.toUpperCase() == 'XMLSELECT' && attrname == 'search')
                return false;
            if (parentName == 'ReposDef' && attrname == 'base')
                return false;
            if (parentName == 'UserContext' && attrname == 'userid')
                return false;
        }
        else
        {
            if (grandParentName == 'Enumeration' && parentName == 'Value')
                return false;
            if (grandParentName == 'Configuration' && 
                    parentName == 'EnumeratedValues')
                return false;
        }
        return true;
    }

    // Return true if the given line from a multi-line TextStore StoredText
    // value looks like it contains a file or directory path, false
    // otherwise.
    def lineHasPath(obj, line)
    {
        if (!hasSlash(line))
            return false;
        def m = (line =~ /^\s*([^=]+)\s*=\s*(.*)$/);
        if (m.find())
        {
            def key = m.group(1);
            def value = m.group(2);

            // Exclude specific key patterns that point to metadata paths
            if (key == 'oma.folder.products.path')
                return false;
            if (key.startsWith('installed.object.stpsamples.'))
                return false;
            if (key.startsWith('data.') && key.endsWith('.path'))
                return false;

            // Exclude keys that indicate DAV URLs
            if (key in [
                    'iomsrv.httpserver.basepath.dir', 
                    'iomsrv.httpserver.contentmap.dir',
                    ])
                return false;

            // Exclude user IDs
            if (key.endsWith('.userid'))
                return false;

            // Exclude HTTP URLs
            if (key.endsWith('.url'))
                return false;
            if (value.startsWith('http:'))
                return false;
            if (key == 'stp.sasweb.dir')
                return false;

            // Exclude JNDI context roots
            if (key.endsWith('.webservice.contextroot'))
                return false;

            // Miscellanous exclusions of keys whose values may contain
            // slashes but they aren't files or directories
            if (key in [
                    'webapp.biws.maker.deployment.target.list', // "N/A"
                    ])
                return false;
        }
        return true;
    }
 
    /////////////////////////////////////////////////////////////////////
    // Begin overridable methods

    def handleStart()
    {
    }

    def handleObj(obj, attrnames)
    {
        this.objectHadPath = false;
        attrnames.each
        { attrname ->
            if (isPathLike(obj, attrname))
            {
                handleObjAttr(obj, attrname);
            }
        }
        if (this.objectHadPath)
            this.objectsWithPathCount += 1;
    }

    def handleObjAttr(obj, attrname)
    {
        def typename = obj.getTypeName();
        if (typename == 'TextStore' && attrname == 'StoredText')
        {
            handleStoredText(obj);
        }
        else
        {
            this.objectHadPath = true;
            def name = obj.getName();
            def attrval = obj.getAttribute(attrname);
            handleAttrVal(obj, name, attrname, attrval);
        }
    }

    def handleRepository(repos)
    {
    }

    def handleType(typename)
    {
    }

    def handleAttrVal(obj, name, attrname, attrval)
    {
    }

    def handleStoredText(obj)
    {
        def text = obj.getAttribute('StoredText');
        if (text == null)
            return null;
        if (isXML(text))
            return handleXML(obj, text);
        if (text.indexOf('\n') < 0)
        {
            this.objectHadPath = true;
            return handleAttrVal(obj, obj.getName(), 'StoredText', text);
        }
        return handleMultiLineText(obj, text);
    }

    // Process an XML text object.
    // Return the XML DOM Document object parsed from the text.
    // Return null if parsing failed or the XML document is a template and
    // should not be modified.
    def handleXML(obj, text)
    {
        if (text == null)
            return null;

        // Template XML should not be modified
        if (isTemplateTextStore(obj))
            return null;

        def nodeValueCallback = 
        { parent, node, value ->
            if (nodeHasPath(parent, node, value))
            {
                this.objectHadPath = true;
                handleXMLValue(obj, parent, node, value);
                return true;
            }
            return false;
        }

        def doc = buildDOMTree(text);
        if (doc == null)
            return null; // Not really XML
        traverseDOMNodes(doc, nodeValueCallback);
        return doc;
    }

    def handleXMLValue(obj, parent, node, value)
    {
    }

    def handleMultiLineText(obj, text)
    {
        def lines = text.split('\n');
        def lineList = lines.collect
        { line -> 
            if (lineHasPath(obj, line))
            {
                this.objectHadPath = true;
                return handleTextLine(obj, line);
            }
            else
            {
                return line;
            }
        }
        return lineList;
    }

    // Return the filtered line value
    def handleTextLine(obj, line)
    {
        return line;
    }

    // End overridable methods
    /////////////////////////////////////////////////////////////////////
}
