package com.sas.solutions.brm.migrate;

import com.sas.solutions.brm.util.BRMUtil;

public class TextStorePropDumper extends OMRVisitorBase
{
    def tempDir = null;
    def logger = null;

    public TextStorePropDumper(omr, tempDir, logger)
    {
        super(omr);
        this.tempDir = tempDir;
        this.logger = logger;
    }

    def dumpComponentsWithTextStores()
    {
        def componentsDir = new File(tempDir, 'components-with-textstores');
        this.logger.info("Dumping components with TextStore objects containing configuration properties to directory: ${componentsDir}");
        componentsDir.mkdirs();
        def searchParam = "*[Notes/TextStore[@Name='Public Configuration Properties' OR @Name='Private Configuration Properties']]";
        def matchExactType = false;
        omr.doOMRQuery('SoftwareComponent', searchParam, matchExactType).each
        { obj ->
            if (obj.getName() == 'Deployment Tester Server')
                return;
            def classId = obj.getAttribute('ClassIdentifier');
            if (classId == 'levdirroot')
                return;
            dumpComponentAndNotes(omr, componentsDir, obj);
        }
    }

    def dumpComponentAndNotes(omr, componentsDir, obj)
    {
        def componentDir = new File(componentsDir, obj.getId());
        componentDir.mkdir();
        def objFile = new File(componentDir, obj.getId() + '.xml');
        BRMUtil.writeStringToFile(objFile, obj.getXML());
        def notes = obj.getAssociation('Notes');
        if (notes == null)
            notes = [];
        notes.each
        { note ->
            def noteId = note.getId();
            note = omr.doOMRQuery('TextStore', "*[@Id='${noteId}']", true).get(0);
            BRMUtil.writeStringToFile(
                new File(componentDir, note.getName() + '.xml'), 
                note.getXML());
        }
    }

    def dumpDeploymentTesterServers()
    {
        def deploymentTesterDir = new File(tempDir, 'deployment-tester');
        this.logger.info("Dumping Deployment Tester Server configuration to directory: ${deploymentTesterDir}");
        deploymentTesterDir.mkdirs();
        def searchParam = "@Name='Deployment Tester'";
        def objs = omr.doOMRQuery('ServiceComponent', searchParam, true);
        if (objs.size() > 0)
        {
            def deploymentTester = objs[0];
            def objFile = new File(deploymentTesterDir, 'Deployment Tester.xml');
            BRMUtil.writeStringToFile(objFile, deploymentTester.getXML());
        }
        searchParam = "@Name='Deployment Tester Server'";
        omr.doOMRQuery('ServiceComponent', searchParam, true).each
        { obj ->
            dumpDeploymentTesterServer(deploymentTesterDir, obj);
        }
    }

    def dumpDeploymentTesterServer(dumpDir, serverObj)
    {
        def serverDir = new File(dumpDir, serverObj.getId());
        serverDir.mkdir();
        def objFile = new File(serverDir, 'ServiceComponent.xml');
        BRMUtil.writeStringToFile(objFile, serverObj.getXML());
        def connectionObj = serverObj.getAssociatedObject('SourceConnections');
        if (connectionObj != null)
        {
            connectionObj = getObject(connectionObj);
            objFile = new File(serverDir, 'TCPIPConnection.xml');
            BRMUtil.writeStringToFile(objFile, connectionObj.getXML());
        }
        def notes = serverObj.getAssociation('Notes');
        if (notes == null)
            notes = [];
        notes.each
        { note ->
            def noteId = note.getId();
            note = getObject(note);
            BRMUtil.writeStringToFile(
                new File(serverDir, note.getName() + '.xml'), 
                note.getXML());
        }
    }

    def dumpLevDirRootComponents()
    {
        // Dump levdirroot components (there is 1 per machine)
        def levDirRootDir = new File(tempDir, 'levdirroot');
        this.logger.info("Dumping LevDirRoot components (1 per machine) to directory: ${levDirRootDir}");
        levDirRootDir.mkdirs();
        def searchParam = "*[@ClassIdentifier='levdirroot'][Notes/TextStore[@Name='Public Configuration Properties' OR @Name='Private Configuration Properties']]";
        def matchExactType = false;
        omr.doOMRQuery('SoftwareComponent', searchParam, matchExactType).each
        { obj ->
            dumpComponentAndNotes(omr, levDirRootDir, obj);
        }
    }

    def dumpOrphanTextStores()
    {
        // Dump orphan TextStore objects
        def orphansDir = new File(tempDir, 'orphan-textstores');
        this.logger.info("Dumping \"orphan\" TextStore objects containing configuration properties to directory: ${orphansDir}");
        orphansDir.mkdir()
        def searchParam = "*[@Name='Public Configuration Properties' OR @Name='Private Configuration Properties']";
        def matchExactType = true;
        omr.doOMRQuery('TextStore', searchParam, matchExactType).each
        { textstore ->
            def objs = textstore.getAssociation('Objects');
            if (objs != null && objs.size() > 0)
                return;
            def textStoreFile = new File(orphansDir, textstore.getId() + '.xml');
            BRMUtil.writeStringToFile(textStoreFile, textstore.getXML());
        }
    }

    def dump()
    {
        this.logger.info("Dumping TextStore objects containing configuration properties");
        this.omr.setFoundationRepository();
        dumpComponentsWithTextStores();
        dumpDeploymentTesterServers();
        dumpLevDirRootComponents();
        dumpOrphanTextStores();
    }
}
