/*
 * Created on June 30, 2010
 * Copyright (c) 2010 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.migrate;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import com.sas.solutions.brm.errors.BRMInternalError;
import com.sas.solutions.brm.omr.GenericMetadataObject;

class VisitorBase
{
    public static boolean isSAS(obj, String s)
    {
        if (obj != null)
        {
            def textrole = obj.getAttribute('TextRole');
            if (textrole != null)
            {
                if (textrole.toLowerCase() == 'sas code')
                    return true;
            }
        }
        // Text enclosed in /* ... */ comment markers
        if (s =~ /(?s)^\s*\/\*.*\*\/\s*$/)
            return true;
        // Text beginning with %word and ending with ;
        if (s =~ /(?s)(?m)^\s*%[A-Za-z]+.*;\s*$/)
            return true;
        return false;
    }

    public static boolean isXML(String s)
    {
        if (s =~ /^<\?xml.*\?>/)
            return true;
        if (s =~ /^\s*<[A-Za-z]+.*>/)
            return true;
        return false;
    }
    
    public static String domTreeToString(Document doc)
    {
        StringWriter sw = new StringWriter();
        try
        {
              OutputFormat format = new OutputFormat(doc);
              XMLSerializer output = new XMLSerializer(sw, format);
              output.serialize(doc);
              return sw.toString();
        }
        catch (IOException e) {
            throw new BRMInternalError(e);
        }
    }
    
    public static Document buildDOMTree(String inStr)
    {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setValidating(false);
        try
        {
            DocumentBuilder db = dbf.newDocumentBuilder();
            return db.parse(new ReaderToInputStream(new StringReader(inStr)));
        }
        catch (ParserConfigurationException e)
        {
            throw new BRMInternalError(e);
        }
        catch (SAXException e)
        {
            // The input string was not well-formed XML
            return null;
        }
        catch (IOException e)
        {
            throw new BRMInternalError(e);
        }
    }

    // Recurse through the attribute and text values on a DOM Document node
    // Return the number of times valueCallback returns true
    // node: A org.w3c.dom.Node object
    // valueCallback:
    //      Closure taking parameters: (parent, node, value)
    //      Called for each ATTRIBUTE_NODE or TEXT_NODE, or CDATA_SECTION_NODE
    //      Should return true if search condition is true, false otherwise
    def static traverseDOMNodes(Node node, Closure valueCallback)
    {
        def findCount = 0;
        def nodes = node.getChildNodes();
        for (def i=0; i<nodes.getLength(); ++i)
        {
            def child = nodes.item(i);
            def nodeType = child.getNodeType();
            if (nodeType == Node.ELEMENT_NODE)
            {
                //println("Element: ${child.getNodeName()}");
                def attrs = child.getAttributes();
                for (def j=0; j<attrs.getLength(); j++)
                {
                    def attr = attrs.item(j);
                    if (valueCallback(child, attr, attr.getNodeValue()))
                        findCount += 1;
                    //println("AttrName: ${attr.getNodeName()} AttrValue: ${attr.getNodeValue()}");
                }
                findCount += traverseDOMNodes(child, valueCallback);
            }
            else if (nodeType == Node.CDATA_SECTION_NODE ||
                     nodeType == Node.TEXT_NODE)
            {
                if (valueCallback(node, child, child.getNodeValue()))
                    findCount += 1;
                //println("Text: ${child.getNodeValue()}");
            }
        }
        return findCount;
    }

    def static isTemplateTextStore(GenericMetadataObject obj)
    {
        def textRole = obj.getAttribute("TextRole");
        if (textRole != null && textRole.toUpperCase().equals("TEMPLATE"))
            return true;
        def text = obj.getAttribute('StoredText');
        if (text != null && text.startsWith('<Template '))
            return true;
        return false;
    }

    def static guessShortName(hostname)
    {
        if (hostname == null)
            return null;
        hostname = hostname.trim().toUpperCase();
        if (hostname.length() == 0)
            return "";
        def i = hostname.indexOf('.');
        if (i < 0)
            return hostname;
        return hostname.substring(0, i);
    }
}
