/*
 * Created on Aug 24, 2006
 * Copyright (c) 2006 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.omr;

import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sas.services.connection.BridgeServer;
import com.sas.services.connection.ConnectionFactoryConfiguration;
import com.sas.services.connection.ConnectionFactoryException;
import com.sas.services.connection.ConnectionFactoryInterface;
import com.sas.services.connection.ConnectionFactoryManager;
import com.sas.services.connection.ConnectionInterface;
import com.sas.services.connection.ManualConnectionFactoryConfiguration;

class LoggerLevelSetter implements PrivilegedAction
{
    String factoryName;
    Level level;

    public LoggerLevelSetter(String factoryName, Level level)
    {
        this.factoryName = factoryName;
        this.level = level;
    };

    public Object run()
    {
        Logger logger = Logger.getLogger(factoryName);
        logger.setLevel(level);
        return null;
    }
}

public class OMRConnector
{
    /**
     * This makes a connection to the omr server with given params.
     *
     * @param cmrHost Name of host to connect to.
     * @param cmrPort Port on host to use.
     * @param cmrUserName Username on host.
     * @param cmrPassword Password for username.
     * @return The "bridge" connection object. Call close() when done.
     * @throws OMRException if a connection cannot be made
     * @see com.sas.metadata.MetadataWorkspace
     */
    public static ConnectionInterface makeConnection(
            String cmrHost, String cmrPort,
            String cmrUserName, String cmrPassword)
        throws OMRException
    {
        BridgeServer omrServer = new BridgeServer(BridgeServer.CLSID_SASOMI, 
                cmrHost, Integer.parseInt(cmrPort));
        ConnectionFactoryConfiguration cxfConfig = 
            new ManualConnectionFactoryConfiguration(omrServer);

        ConnectionInterface cx = null;
        try
        {
            ConnectionFactoryInterface cxf = 
                ConnectionFactoryManager.getConnectionFactory(cxfConfig);
            String factoryName = 
                ConnectionFactoryManager.getConnectionFactoryLoggerName(
                        cxfConfig);
            // Turn off annoying trace messages before connecting
            configLogger(factoryName, Level.OFF);
            cx = cxf.getConnection(cmrUserName, cmrPassword);
        }
        catch (ConnectionFactoryException e)
        {
            throw new OMRException(e);
        }
        return cx;
    }

    public static void configLogger(String factoryName, final Level level)
    {

        // Need java.util.logging.LoggingPermission("control")
        // to do this. Otherwise, we'll have to leave things alone.
        LoggerLevelSetter action = new LoggerLevelSetter(factoryName, level);
        try
        {
            AccessController.doPrivileged(action);
        }
        catch (SecurityException se)
        {
            // ignore
        }
    }
}
