/*
 * Created on Jun 23, 2006
 * Copyright (c) 2006 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.omr;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Iterator;

public class OMRUtil
{
    private OMRUtil()
    {
        // a class with nothing but static utility methods
    }

    /**
     * Escape XML special characters.
     * This method differs from BRMUtil.escapeXML() in that it escapes the
     * tab, carriage return and newline characters also.
     */
    public static String escapeXML(String s)
    {
        s = s.replaceAll("&", "&amp;");
        s = s.replaceAll("\"", "&quot;");
        s = s.replaceAll("<", "&lt;");
        s = s.replaceAll(">", "&gt;");
        s = s.replaceAll("\\t", "&#x09;");
        s = s.replaceAll("\\n", "&#x0a;");
        s = s.replaceAll("\\r", "&#x0d;");
        return s;
    }


    /**
     * Load a GenericMetadataObject from an xml file.
     * The XML file is in the same format as the string produced by the
     * getXML() method on GenericMetadataObject.
     */
    public static GenericMetadataObject loadObjectFromFile(File objFile)
    {
        if (!objFile.isFile())
        {
            return null;
        }
        StringBuffer sb = new StringBuffer();
        char[] buf = new char[256];
        BufferedReader br = new BufferedReader(
                                new InputStreamReader(
                                    new FileInputStream(objFile), 
                                    "UTF-8"));
        while (true)
        {
            int n = br.read(buf);
            if (n == -1)
                break;
            sb.append(buf, 0, n);
        }
        br.close();
        ObjectInfoHandler infoHandler = ObjectInfoHandler.parseXML(
                                                            sb.toString());
        return new GenericMetadataObject(
                                    infoHandler.getObjectInfo());
    }

    /**
     * Copy attributes from srcObj to dstObj.
     * The attributes Id, MetadataCreated, and MetadataUpdated are skipped.
     */
    public static void copyObjectAttributes(GenericMetadataObject srcObj, 
                                            GenericMetadataObject dstObj)
    {
        Iterator iterator = srcObj.getAttributeNames().iterator();
        while (iterator.hasNext())
        {
            String attrname = (String)iterator.next();
            if (attrname.equals("Id") ||
                attrname.equals("MetadataCreated") ||
                attrname.equals("MetadataUpdated"))
            {
                continue;
            }
            dstObj.setAttribute(attrname, srcObj.getAttribute(attrname));
        }
    }

    // Return a copy of the GenericMetadataObject with all associations
    // and the ID removed. No change is made to metadata.
    public static GenericMetadataObject stripAssocs(GenericMetadataObject obj)
    {
        def newObj = new GenericMetadataObject();
        newObj.setTypeName(obj.getTypeName());
        copyObjectAttributes(obj, newObj);
        return newObj;
    }
}
