/*
 * Created on Apr 12, 2006
 * Copyright (c) 2006 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.omr;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;


/**
 * This SAX Handler class is for parsing the result of the IOMI 
 * GetMetadataObjects() method.
 * @author davhan
 */
public class ObjectListHandler extends BaseOMRXMLHandler
{
    private static final int STATE_INIT = 0;
    private static final int STATE_IN_LIST = 1;
    private static final int STATE_AFTER_LIST = 2;
    
    private String listTag;
    private int state;
    private ObjectInfoHandler handler;
    private List listItems;

    public ObjectListHandler()
    {
        this(null);
    }

    public ObjectListHandler(String listTag)
    {
        super();
        this.listTag = listTag;
        this.state = STATE_INIT;
        this.handler = null;
        this.listItems = new ArrayList();
    }

    public String getListTag()
    {
        return listTag;
    }
    
    public List getList()
    {
        return this.listItems;
    }
    
    public void startElement(String uri, String localName, String qName,
            Attributes attributes) throws SAXException 
    {
        if (this.state == STATE_INIT)
        {
            if (this.listTag == null || this.listTag.equalsIgnoreCase(qName))
            {
                this.listTag = qName;
                this.state = STATE_IN_LIST;
            }
            return;
        }
        if (this.state == STATE_IN_LIST)
        {
            if (this.handler == null)
                this.handler = new ObjectInfoHandler();
            this.handler.startElement(uri, localName, qName, attributes);
            return;
        }
        if (this.state == STATE_AFTER_LIST)
            return;
        throw new SAXException("Unknown state: ${this.state}");
    }
    
    public void endElement(String uri, String localName, String qName)
            throws SAXException 
    {
        if (this.state == STATE_INIT)
            return;
        if (this.state == STATE_IN_LIST)
        {
            if (this.handler != null)
            {
                this.handler.endElement(uri, localName, qName);
                if (this.handler.getState() == 
                        ObjectInfoHandler.STATE_AFTER_LIST)
                {
                    this.listItems.add(this.handler.getObjectInfo());
                    this.handler = null;
                }
            }
            else
            {
                this.state = STATE_AFTER_LIST;
            }
            return;
        }
        if (this.state == STATE_AFTER_LIST)
            return;
        throw new SAXException("Unknown state: ${this.state}");
    }
    
    public static ObjectListHandler parseXML(String inputXML)
    {
        ObjectListHandler handler = new ObjectListHandler();
        parseXML(handler, inputXML);
        return handler;
    }
    
    public static ObjectListHandler parseXML(String inputXML, 
                                             String listTag)
    {
        ObjectListHandler handler = new ObjectListHandler(listTag);
        parseXML(handler, inputXML);
        return handler;
    }

    /**
     * @param args
     */
    public static void main(String[] args)
    {
        String inputXML = "<Objects><ConfiguredComponent Id=\"A557SURL.AH00099E\" Name=\"OMR Tier\" ChangeState=\"\" ClassIdentifier=\"SSS_OMR_TIER\" Desc=\"SAS Solutions Metadata Server Tier\" InstallationLocation=\"\" IsLicensed=\"0\" LockedBy=\"\" Major=\"1\" MetadataCreated=\"04Apr2006:19:58:42\" MetadataUpdated=\"04Apr2006:19:58:42\" Minor=\"1\" Platform=\"windows\" ProductIdentifier=\"\" ProductName=\"SAS Solutions\" SoftwareVersion=\"1.1\" SpecVendor=\"\" SpecVersion=\"\" Vendor=\"SAS\" VendorIdentifier=\"\"/><ConfiguredComponent Id=\"A557SURL.AH00099F\" Name=\"MySQL Tier\" ChangeState=\"\" ClassIdentifier=\"SSS_MYSQL_TIER\" Desc=\"SAS Solutions MySQL Tier\" InstallationLocation=\"\" IsLicensed=\"0\" LockedBy=\"\" Major=\"1\" MetadataCreated=\"04Apr2006:19:58:44\" MetadataUpdated=\"04Apr2006:19:58:44\" Minor=\"1\" Platform=\"windows\" ProductIdentifier=\"\" ProductName=\"SAS Solutions\" SoftwareVersion=\"1.1\" SpecVendor=\"\" SpecVersion=\"\" Vendor=\"SAS\" VendorIdentifier=\"\"/><ConfiguredComponent Id=\"A557SURL.AH00099J\" Name=\"SSS Data Tier\" ChangeState=\"\" ClassIdentifier=\"SSS_Data_TIER\" Desc=\"SAS Solutions Services Data Tier\" InstallationLocation=\"\" IsLicensed=\"0\" LockedBy=\"\" Major=\"1\" MetadataCreated=\"04Apr2006:19:59:17\" MetadataUpdated=\"04Apr2006:19:59:17\" Minor=\"1\" Platform=\"windows\" ProductIdentifier=\"\" ProductName=\"SAS Solutions\" SoftwareVersion=\"1.1\" SpecVendor=\"\" SpecVersion=\"\" Vendor=\"SAS\" VendorIdentifier=\"\"/><ConfiguredComponent Id=\"A557SURL.AH00099N\" Name=\"SSS Middle Tier\" ChangeState=\"\" ClassIdentifier=\"SSS_MIDDLE_TIER_PRIMARY\" Desc=\"SAS Solutions Services Middle Tier\" InstallationLocation=\"\" IsLicensed=\"0\" LockedBy=\"\" Major=\"1\" MetadataCreated=\"04Apr2006:20:00:08\" MetadataUpdated=\"04Apr2006:20:00:08\" Minor=\"2\" Platform=\"windows\" ProductIdentifier=\"\" ProductName=\"SAS Solutions\" SoftwareVersion=\"1.2\" SpecVendor=\"\" SpecVersion=\"\" Vendor=\"SAS\" VendorIdentifier=\"\"/></Objects>";
        ObjectListHandler listHandler = parseXML(inputXML);
        System.out.println("List tag: " + listHandler.getListTag());
        List itemList = listHandler.getList();
        for (Iterator it=itemList.iterator(); it.hasNext(); )
        {
            Map item = (Map)it.next();
            System.out.println("Item: " + item);
        }
    }

}
