/*
 * Created on Jan. 29, 2010
 * Copyright (c) 2010 by SAS Institute Inc., Cary, NC 27513
 */
package com.sas.solutions.brm.util;

import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Echo;

import com.sas.solutions.brm.logging.BRMBuildListener;

/**
 * Static utility methods to be called from Groovy scripts in Ant XML files.
 */
public class BRMAntUtil
{
    /**
     * @param msg  The message to be output
     * @param level  String containing the level name, i.e. "debug", "info",
     * "warn", "error".
     */
    public static void echo(Binding b, String msg, String level)
    {
        def project = b.getProperty("project");
        def self = b.getProperty("self");
        def echoTask = project.createTask("echo");
        //echoTask.setOwningTarget(self.getOwningTarget());
        echoTask.bindToOwner(self);
        echoTask.setMessage(msg);
        def echoLevel = new Echo.EchoLevel();
        echoLevel.setValue(level);
        echoTask.setLevel(echoLevel);
        echoTask.perform();
    }

    public static void debug(Binding b, String msg)
    {
        echo(b, msg, "debug");
    }

    public static void info(Binding b, String msg)
    {
        echo(b, msg, "info");
    }

    public static void warn(Binding b, String msg)
    {
        echo(b, msg, "warn");
    }

    public static void error(Binding b, String msg)
    {
        echo(b, msg, "error");
    }

    // Run a program via an Ant task, throw exception on non-zero exit code
    // executable = name or path to program
    // args = List of String parameters to program
    public static void exec(Binding b, Object executable, Collection args,
                            Object failonerror)
    {
        def project = b.getProperty("project");
        def self = b.getProperty("self");
        def exec = project.createTask("exec");
        exec.setOwningTarget(self.getOwningTarget());
        if (executable instanceof File)
            exec.setExecutable(executable.getPath());
        else
            exec.setExecutable(executable);
        def argstr = null;
        def arg = null;
        def it = null;
        for (it=args.iterator(); it.hasNext(); )
        {
            argstr = it.next();
            arg = exec.createArg();
            arg.setValue(argstr);
        }
        if (failonerror instanceof String)
            exec.setFailonerror(BRMUtil.isTrueStr(failonerror));
        else
            exec.setFailonerror(failonerror);
        exec.perform();
    }

    public static String getPropertyOrDie(Project project, String propname)
    {
        String propval = project.getProperty(propname);
        if (propval == null)
        {
            throw new Exception("Missing property \"${propname}\"");
        }
        return propval;
    }

    public static void runTask(Binding b, String taskname, Map params)
    {
        def project = b.getProperty("project");
        def self = b.getProperty("self");
        def task = project.createTask(taskname);
        task.bindToOwner(self);
        params.each { name, value ->
            task.setDynamicAttribute(name.toLowerCase(), value);
        }
        task.perform();
    }

    public static BRMDBUtil getDBUtilFromProject(Project project)
    {
        def logger = BRMBuildListener.getLogger(project);
        def dbutil = new BRMDBUtil(project.getProperties(), logger);
        return dbutil;
    }
}
