package com.sas.solutions.brm.webservice;

class GroovyWebServiceMaker {
	public static final String METADATA_NAME = "WebServiceMaker";
	public static final String REGISTERED_SAS_APPLICATION_IDENTIFIER = "Registered SAS Application";
	public static final String BIWEBSERVICES_JAVA_PREFIX = "BI Web Services for Java";
	
	protected com.sas.metadata.remote.DeployedComponent deployedComponent;
	protected URL endpoint;
	
	public GroovyWebServiceMaker(com.sas.metadata.remote.DeployedComponent deployedComponent) {
		this.deployedComponent = deployedComponent;
	}
	
	public GroovyWebServiceMaker(URL endpoint) {
		this.endpoint = endpoint;
	}
	
	public URL getEndpoint() throws java.rmi.RemoteException, com.sas.metadata.remote.MdException {
		if (endpoint == null && deployedComponent != null) {
			// see if we have a connection on this maker here, if so, use that
			com.sas.metadata.remote.TCPIPConnection tcpipConnection = null;
			boolean useJavaInference = false;
			int numConnections = deployedComponent.getSourceConnections().size();
			if (deployedComponent.getSourceConnections() != null && numConnections > 0) {
				for (Iterator it = deployedComponent.getSourceConnections().iterator(); it.hasNext();) {
					Object connection = it.next();
					if (connection instanceof com.sas.metadata.remote.TCPIPConnection) {
						tcpipConnection = (com.sas.metadata.remote.TCPIPConnection)connection;
						break;
					}
				}
			} else {
				// no source connection here, so we'll need to infer that from the Registered SAS Application DeployedComponent
				com.sas.metadata.remote.SoftwareComponent parentSC = deployedComponent.getDescriptiveComponent();
				if (parentSC != null && parentSC.getName().startsWith(BIWEBSERVICES_JAVA_PREFIX)) {
					for (Iterator it = parentSC.getDeployedComponents().iterator(); it.hasNext();) {
						com.sas.metadata.remote.DeployedComponent dc = (com.sas.metadata.remote.DeployedComponent)it.next();
						if (dc.getName().equals(REGISTERED_SAS_APPLICATION_IDENTIFIER)) {
							if (dc.getSourceConnections() != null) {
								for (Iterator scIt = dc.getSourceConnections().iterator(); it.hasNext();) {
									Object connection = scIt.next();
									if (connection instanceof com.sas.metadata.remote.TCPIPConnection) {
										tcpipConnection = (com.sas.metadata.remote.TCPIPConnection)connection;
										useJavaInference = true;
									}
									break;
								}
							}
							if (tcpipConnection != null)
								break;
						}
					}
				}
			}
			if (tcpipConnection != null) {
				try {
					String path = tcpipConnection.getService();
					if (useJavaInference) {
						if (!path.endsWith("/"))
							path += "/";
						path += "services/WebServiceMaker";
					}
					endpoint = new URL(tcpipConnection.getCommunicationProtocol(), tcpipConnection.getHostName(), tcpipConnection.getPort(), path);
				} catch (MalformedURLException e) {}
			}
		}
		return endpoint;
	}
	
	/**
	 * Delete a service using the supplied metadata credentials.
	 * @param serviceName The name of the service to delete.
	 * @param username The username that will be authenticated in metadata.
	 * @param password The password associated with the username.
	 * @throws MdException 
	 * @throws RemoteException 
	 */
	public void deleteService(String serviceName, String username, String password) throws java.rmi.RemoteException, com.sas.metadata.remote.MdException, org.apache.axis2.AxisFault {
		GroovyWebServicesClient client = new GroovyWebServicesClient(getEndpoint(), GroovyWebServicesClient.REMOVE_WEB_SERVICE_ACTION_92, username, password);
		org.apache.axiom.om.OMElement payload = GetRemoveWebServiceElement(serviceName);
		client.sendReceive(payload);
	}
	
	/**
	 * Delete a service using the userid and password of the current user context. This method uses a OTP.
	 * @param serviceName The name of the service to delete.
	 * @throws ServiceException
	 * @throws RemoteException
	 * @throws MdException 
	 */
	public void deleteService(String serviceName, com.sas.services.user.UserContextInterface userContext) throws com.sas.services.ServiceException, java.rmi.RemoteException, com.sas.metadata.remote.MdException {
		Credentials creds = GetOTP(userContext);
		deleteService(serviceName, creds.username, creds.password);
	}
	
	public URL makeWebService(String webServiceName, String[] stps, String namespace, String[] keywords, String username, String password, boolean createMetadata) throws java.rmi.RemoteException, com.sas.metadata.remote.MdException, org.apache.axis2.AxisFault, MalformedURLException {
		if (stps == null || stps.length < 1)
			throw new IllegalArgumentException("You must include stored processes to create a web service");
		
		GroovyWebServicesClient client = new GroovyWebServicesClient(getEndpoint(), GroovyWebServicesClient.MAKE_WEB_SERVICE_ACTION_92, username, password);
		org.apache.axiom.om.OMElement payload = getMakeWebServiceElement(webServiceName, stps, namespace, keywords, createMetadata);
		org.apache.axiom.om.OMElement result = client.sendReceive(payload);
		if(result != null)
		{
			org.apache.axiom.om.OMElement elem = (org.apache.axiom.om.OMElement)result.getFirstElement();
							    				
			String wsUrl = elem.getText();
			
			if((wsUrl != null) && (wsUrl.length() > 0))
			{
				return new URL(wsUrl);
			}
			else
			{
				// no axis fault, but no URL returned
				// probably from Java in Directory mode which will not deploy the new service but instead dump it
				// to a directory. so return null here and let the caller decide what to do
				return null;
			}
		} else {
			throw new org.apache.axis2.AxisFault("Web Service Maker did not return a result");
		}
	}
	
	public URL makeWebService(String webServiceName, String[] stps, String namespace, String[] keywords, com.sas.services.user.UserContextInterface userContext, boolean createMetadata) throws java.rmi.RemoteException, com.sas.metadata.remote.MdException, org.apache.axis2.AxisFault, MalformedURLException {
		if (stps == null || stps.length < 1)
			throw new IllegalArgumentException("You must include stored processes to create a web service");
		
		Credentials creds = GetOTP(userContext);
		return makeWebService(webServiceName, stps, namespace, keywords, creds.username, creds.password, createMetadata);
	}
	
	public List listWebServices(com.sas.services.user.UserContextInterface userContext) throws org.apache.axis2.AxisFault, java.rmi.RemoteException, com.sas.metadata.remote.MdException, com.sas.services.ServiceException {
		List retList = new ArrayList();
		Credentials creds = GetOTP(userContext);
		
		GroovyWebServicesClient client = new GroovyWebServicesClient(getEndpoint(), GroovyWebServicesClient.LIST_WEB_SERVICES_ACTION_92, creds.username, creds.password);
		org.apache.axiom.om.OMElement payload = getListWebServicesElement();
		org.apache.axiom.om.OMElement result = client.sendReceive(payload);
		if(result != null)
		{
			org.apache.axiom.om.OMElement listResult = (org.apache.axiom.om.OMElement) result.getFirstElement();
			for(Iterator children = listResult.getChildren(); children.hasNext();)
			{
				org.apache.axiom.om.OMElement elem = (org.apache.axiom.om.OMElement)children.next();
				if(elem.getLocalName().equals("string"))
				{
					String serviceName = elem.getText();
					retList.add(serviceName);
				}
			}
		}
		return retList;
	}
	
	public static org.apache.axiom.om.OMElement GetRemoveWebServiceElement(String serviceName) {
		org.apache.axiom.om.OMFactory fac = org.apache.axiom.om.OMAbstractFactory.getOMFactory();
        org.apache.axiom.om.OMNamespace omNs = fac.createOMNamespace(GroovyWebServicesClient.WEB_SERVICES_NS_92, "");
        org.apache.axiom.om.OMElement method = fac.createOMElement("RemoveWebService", omNs);
        org.apache.axiom.om.OMElement serviceNameEl = fac.createOMElement("serviceName", omNs);
        org.apache.axiom.om.OMText serviceNameText = fac.createOMText(serviceNameEl, serviceName);
        serviceNameEl.addChild(serviceNameText);
        method.addChild(serviceNameEl);
        return method;
	}
	
	protected static Credentials GetOTP(com.sas.services.user.UserContextInterface userContext) throws com.sas.services.ServiceException, java.rmi.RemoteException {
		Credentials creds = new Credentials();
		com.sas.services.information.ServerInterface authServer = userContext.getAuthServer();
		com.sas.services.user.UserIdentityInterface id = authServer.generatePassword(null);
		creds.username = id.getPrincipal().toString();
		creds.password = id.getCredential().toString();
		return creds;
	}
	
	/**
	 * Creates a <MakeWebService> element for the WebServiceMaker that is used when Finish is pressed in the wizard.
	 * @return
	 */
	public static org.apache.axiom.om.OMElement getMakeWebServiceElement(String webServiceName, String[] stps, String namespace, String[] keywords, boolean createMetadata)
	{
		org.apache.axiom.om.OMFactory fac = org.apache.axiom.om.OMAbstractFactory.getOMFactory();
        org.apache.axiom.om.OMNamespace omNs = fac.createOMNamespace(GroovyWebServicesClient.WEB_SERVICES_NS_92, "");
        org.apache.axiom.om.OMElement method = fac.createOMElement("MakeWebService", omNs);
        org.apache.axiom.om.OMElement stringElement = null;
        org.apache.axiom.om.OMText stringElementText = null;
        
        // Create storedProcessPaths elements
        org.apache.axiom.om.OMElement storedProcessPaths = fac.createOMElement("storedProcessPaths", omNs);
        for(int i=0; i<stps.length; i++)
        {
        	stringElement = fac.createOMElement("string", omNs);
        	stringElementText = fac.createOMText(stringElement, stps[i]);
        	stringElementText.setOptimize(false);
            stringElement.addChild(stringElementText);
        	storedProcessPaths.addChild(stringElement);
        }
        method.addChild(storedProcessPaths);
        
        // Create serviceName element
        org.apache.axiom.om.OMElement serviceName = fac.createOMElement("serviceName", omNs);
        org.apache.axiom.om.OMText serviceNameText = fac.createOMText(serviceName, webServiceName);
        serviceNameText.setOptimize(false);
        serviceName.addChild(serviceNameText);
        method.addChild(serviceName);
        
        // Create serviceNamespace element
        org.apache.axiom.om.OMElement serviceNamespace = fac.createOMElement("serviceNamespace", omNs);
        if (namespace != null) {
        	org.apache.axiom.om.OMText serviceNamespaceText = fac.createOMText(serviceNamespace, namespace);
        	serviceNamespaceText.setOptimize(false);
        	serviceNamespace.addChild(serviceNamespaceText);
        }
        method.addChild(serviceNamespace);
        
        // Create keyword elements

        org.apache.axiom.om.OMElement keywordsEl = fac.createOMElement("keywords", omNs);
        if (keywords != null) {
        	for (int i = 0; i < keywords.length; i++)
        	{
        		stringElement = fac.createOMElement("string", omNs);
        		stringElementText = fac.createOMText(stringElement, keywords[i]);
        		stringElementText.setOptimize(false);
        		stringElement.addChild(stringElementText);
        		keywordsEl.addChild(stringElement);
        	}
        }
        method.addChild(keywordsEl);
        
        
        // Create publishMetadata element
        org.apache.axiom.om.OMElement publishMetadataEl = fac.createOMElement("publishMetadata", omNs);
        org.apache.axiom.om.OMText publishMetadataText = fac.createOMText(publishMetadataEl, String.valueOf(createMetadata));
        publishMetadataEl.addChild(publishMetadataText);
        method.addChild(publishMetadataEl);
        
        return method;
	}
	
	/**
     * Creates the <ListWebServices> element used to check for name collisions.
     * @return The <ListWebServices> element.
     */
	public static org.apache.axiom.om.OMElement getListWebServicesElement()
	{
		org.apache.axiom.om.OMFactory fac = org.apache.axiom.om.OMAbstractFactory.getOMFactory();
        org.apache.axiom.om.OMNamespace omNs = fac.createOMNamespace(GroovyWebServicesClient.WEB_SERVICES_NS_92, "");
        org.apache.axiom.om.OMElement method = fac.createOMElement("ListWebServices", omNs);
        return method;
	}
}

// end-of-file
