package com.sas.solutions.brm.webservice;

/**
 * GroovyWebServicesClient is a wrapper for an apache axis2 web service client that
 * encapsulates certain properties and behaviors that are common across SAS BI
 * Web Services. This class handles setting security options and ws-addressing
 * settings.
 * 
 * @author zamars
 */
class GroovyWebServicesClient
		extends org.apache.axis2.client.ServiceClient {

	/**
	 * http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2
	 */
	public static final String	WEB_SERVICES_NS_92			= "http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2";

	/**
	 * http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2/ListWebServices
	 */
	public static final String	LIST_WEB_SERVICES_ACTION_92	= "http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2/ListWebServices";

	/**
	 * http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2/MakeWebService
	 */
	public static final String	MAKE_WEB_SERVICE_ACTION_92	= "http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2/MakeWebService";
	
	/**
	 * http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2/RemoveWebService
	 */
	public static final String	REMOVE_WEB_SERVICE_ACTION_92	= "http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2/RemoveWebService";

	/**
	 * Generates the outflow configuration parameter for the service client. The
	 * outflow configuration controls how security for the message is handled as
	 * it is leaving the apache axis2 context. Here, we say add a username token
	 * in plain text and add a timestamp.
	 * 
	 * @param uName
	 *            The username that has permission to make this web service
	 *            call.
	 * @return The OutflowConfiguration Parameter.
	 */
	private org.apache.axis2.description.Parameter getOutflowConfiguration(String uName) {
		org.apache.rampart.handler.config.OutflowConfiguration outflow = new org.apache.rampart.handler.config.OutflowConfiguration();
		outflow.setActionItems("UsernameToken Timestamp");
		outflow.setUser(uName);
		outflow.setPasswordType("PasswordText");
		return outflow.getProperty();
	}

	private org.apache.axis2.client.Options	_options;

	/**
	 * Sets options on the client to commonly-used values.
	 * 
	 * @param endpoint
	 *            The endpoint of the service we're connecting to.
	 * @param action
	 *            The web service action to perform.
	 * @param user
	 *            The user who has permission to perform this web service call.
	 * @param password
	 *            The password associated with the user.
	 * @param timeoutInMillis
	 *            The socket and client connection time to wait before failing
	 *            in milliseconds.
	 * @throws AxisFault
	 *             If there is a problem engaging WS-Addressing or WS-Security
	 *             modules.
	 */
	private void setOptions(String endpoint, String action, String user, String password, int timeoutInMillis) throws org.apache.axis2.AxisFault {
		Integer timeout = new Integer(timeoutInMillis);
		_options = (this.getOptions() == null ? new org.apache.axis2.client.Options()
				: this.getOptions());
		_options.setTo(new org.apache.axis2.addressing.EndpointReference(endpoint));
		_options.setAction(action);
		_options.setTransportInProtocol(org.apache.axis2.Constants.TRANSPORT_HTTP);
		_options.setSoapVersionURI(org.apache.axis2.Constants.URI_SOAP11_ENV);
		// turn off chunking to see if it fixes a .NET problem
		_options.setProperty(org.apache.axis2.transport.http.HTTPConstants.CHUNKED, "false");
		_options.setProperty(org.apache.axis2.Constants.Configuration.ENABLE_MTOM, org.apache.axis2.Constants.VALUE_FALSE);
		_options.setProperty(org.apache.axis2.transport.http.HTTPConstants.SO_TIMEOUT, timeout);
		_options.setProperty(org.apache.axis2.transport.http.HTTPConstants.CONNECTION_TIMEOUT, timeout);
		_options.setProperty(org.apache.axis2.description.WSDL2Constants.ATTRIBUTE_MUST_UNDERSTAND, "0");
		this.setOptions(_options);
		
		if (user != null && password != null) {
			// fix for .NET axis incompatibility (different year ns)
			_options.setProperty(org.apache.axis2.addressing.AddressingConstants.WS_ADDRESSING_VERSION, org.apache.axis2.addressing.AddressingConstants.Submission.WSA_NAMESPACE);
			_options.setProperty(org.apache.ws.security.handler.WSHandlerConstants.PW_CALLBACK_REF, new PWCBHandler(user, password));
			_options.setProperty(org.apache.ws.security.handler.WSHandlerConstants.USER, user);
			_options.setProperty(org.apache.rampart.handler.WSSHandlerConstants.OUTFLOW_SECURITY, getOutflowConfiguration(user));
			this.engageModule(new javax.xml.namespace.QName(org.apache.axis2.Constants.MODULE_ADDRESSING));
			this.engageModule(new javax.xml.namespace.QName("rampart"));
		}
		
	}

	/**
	 * Construct a GroovyWebServicesClient.
	 * 
	 * @param endpoint
	 *            The endpoint of the web service to connect to.
	 * @param action
	 *            The web service action to perform.
	 * @param user
	 *            The user who has permission to perform this web service call.
	 * @param password
	 *            The password associated with the user.
	 * @throws AxisFault
	 *             If there are communication problems or if there is a SOAP
	 *             fault.
	 */
	public GroovyWebServicesClient(URL endpoint, String action, String user, String password) throws org.apache.axis2.AxisFault {
		super();
		setOptions(endpoint.toExternalForm(), action, user, password, 900000);
	}

}

// end-of-file
