package com.sas.solutions.brm.webservice;

class WSMWrapper
{
    Object log;
	GroovyWebServiceMaker webServiceMaker;
	
	WSMWrapper(log, hostStr, portStr, userStr, passwordStr)
	{
        this.log = log;
        log.debug("WSMWrapper(${hostStr}, ${portStr}, ${userStr}, *****)");
		//no url passed in, must retrieve from metadata
        def wsmUrl = getWebServiceMakerURL(log, hostStr, portStr, userStr, 
                                           passwordStr);
        if (wsmUrl == null)
        {
            throw new NoWebServiceMaker(
                        "Could not find WebServiceMaker in metadata.");
        }
        log.debug("calling initialize(${wsmUrl}");
        this.initialize(wsmUrl);
	}
	
	WSMWrapper(Object log, String wsmUrlStr)
	{
        this.log = log;
		log.debug("In constructor via URL. About to initialize.");
		this.initialize(wsmUrlStr);
		log.debug("Finished initializing WSM.");
	}

    def static getWebServiceMakerURL(log, hostStr, portStr, userStr, 
                                     passwordStr)
    {
		log.debug("getWebServicesMakerURL: About to create JOMA connection.");
		//source("C:/Program Files/SAS/SASDeploymentManager/9.2/products/cfgwizard__92210__prt__xx__sp0__1/Utilities/JomaWrapper.groovy");
		def jomaWrapper = new JomaWrapperInternal(log);
		log.debug("Created JOMA connection. Now connecting.");
		jomaWrapper.connect(hostStr, portStr, userStr, passwordStr);
        try
        {
            log.debug("Connected via JOMA. About to getMetadataObjects.");
            String typeStr = "TCPIPConnection";
            //String searchStr = "Source/DeployedComponent[@Name=&apos;Registered SAS Application&apos;]/DescriptiveComponent/SoftwareComponent[@Name=&apos;BI Web Services for Java 9.2&apos;]";
            String searchStr = "Source/DeployedComponent[@Name='Registered SAS Application']/DescriptiveComponent/SoftwareComponent[@Name=:'BI Web Services for Java']";
            def objects = jomaWrapper.getMetadataObjects(typeStr, searchStr);
            log.debug("Got metadata objects.");
            if (objects != null && objects.size() > 0)
            {
                log.debug("Found TCPIPConnection via BIWS for Java.");
                def connection = objects.get(0);
                def hostName = connection.getHostName();
                def hostPort = connection.getPort();
                def hostService = connection.getService();
                def hostProtocol = connection.getCommunicationProtocol();
                String wsmUrl = hostProtocol + "://" + hostName + ":" + hostPort + hostService + "/services/WebServiceMaker";
                log.debug("WSM url: " + wsmUrl);
                return wsmUrl;
            }
            log.debug("WSM not found via Java. Trying generic WSM search. About to getMetadataObjects.");
            searchStr = "Source/DeployedComponent[@Name=&apos;WebServiceMaker&apos;]";
            objects = jomaWrapper.getMetadataObjects(typeStr, searchStr);
            log.debug("Got metadata objects.");
            if (objects != null && objects.size() > 0)
            {
                log.debug("Found connection object via generic WSM search.");
                def connection = objects.get(0);
                def hostName = connection.getHostName();
                def hostPort = connection.getPort();
                def hostService = connection.getService();
                def hostProtocol = connection.getCommunicationProtocol();
                String wsmUrl = hostProtocol + "://" + hostName + ":" + hostPort + hostService;
                log.debug("WSM url: " + wsmUrl);
                return wsmUrl;
            }
        }
        finally
        {
            jomaWrapper.disconnect();
        }
        return null;
    }
	
	void initialize(String wsmUrlStr)
	{
        log.debug("WSMWrapper: initialize ${wsmUrlStr}");
		log.debug(" Creating URL object.");
		java.net.URL wsmUrl = new java.net.URL(wsmUrlStr);
		log.debug("Created URL, now creating underlying common GroovyWebServiceMaker object.");
		webServiceMaker = new GroovyWebServiceMaker(wsmUrl);
        log.debug("webServiceMaker=${webServiceMaker}");
		log.debug("Created underlying GroovyWebServiceMaker object.");
	}
	
	String makeWebService(String serviceNameStr, String[] storedProcesses, String namespaceStr, String[] keywords, String usernameStr, String passwordStr, String publishMetadataStr, String overwriteStr)
	{
		// Default namespace to http://tempuri.org/ if not set
		if (namespaceStr == null)
		{
			namespaceStr = "http://tempuri.org/" + serviceNameStr;
		}
		
		// Default publishing metadata to true if not set
		boolean publishMetadata	= false;
		if (publishMetadataStr != null)
		{
			publishMetadata = Boolean.parseBoolean(publishMetadataStr);
		}
		
		// Default overwriting existing service to false if not set
		boolean overwrite = false;
		if (overwriteStr != null)
		{
			overwrite = Boolean.parseBoolean(overwriteStr);
		}
		
		boolean found = this.listWebServices(usernameStr, passwordStr).any{it.compareToIgnoreCase(serviceNameStr) == 0};
		
		if (found)
		{
			log.info("Web service '$serviceNameStr' already exists");
			if (overwrite)
			{
				log.info("Overwriting '$serviceNameStr' web service");
				this.removeWebService(serviceNameStr, usernameStr, passwordStr);
			}
			else
			{
				return null;
			}
		}
		
		// create web service
		java.net.URL newServiceEndpoint = webServiceMaker.makeWebService(serviceNameStr, storedProcesses, namespaceStr, keywords, usernameStr, passwordStr, publishMetadata);
		if (newServiceEndpoint != null)
		{
			return newServiceEndpoint.toString();
		} else {
			return null;
		}
	}
	
	void removeWebService(String serviceName, String userName, String password)
	{
		webServiceMaker.deleteService(serviceName, userName, password);
	}

    def listWebServicesXML(String userName, String password)
    {
		log.debug("In wsmWrapper.listWebServicesXML. About to create GroovyWebServicesClient.");
        def endpoint = webServiceMaker.getEndpoint();
        log.debug("endpoint=${endpoint}");
		GroovyWebServicesClient client = new GroovyWebServicesClient(endpoint, GroovyWebServicesClient.LIST_WEB_SERVICES_ACTION_92, userName, password);
		log.debug("Created GroovyWebServicesClient. Getting payload.");
		org.apache.axiom.om.OMElement payload = com.sas.web.services.common.WebServiceMaker.getListWebServicesElement();
		log.debug("Got payload. Now send/receive.");
		org.apache.axiom.om.OMElement result = client.sendReceive(payload);
		log.debug("Got result.");
        return result;
    }
	
	List listWebServices(String userName, String password)
	{
		List retList = new ArrayList();
        def result = listWebServicesXML(userName, password);
		if (result != null)
		{
			log.debug("Result was not null. Getting element and text.");
			org.apache.axiom.om.OMElement listResult = (org.apache.axiom.om.OMElement) result.getFirstElement();
			retList = listResult.getChildren().findAll{ it.getLocalName() == "string"}*.getText();
			log.debug("Got arraylist of existing services.");
		}
		log.debug("Returning for wsmWrapper.listWebServices.");
		return retList;
	}
}

// end-of-file
