/*
 * Copyright (c) 2010 by SAS Institute Inc., Cary, NC 27513
 * Created on Oct. 8, 2010
 */
package com.sas.solutions.brm.xml;

import java.io.File;
import java.io.StringReader;
import java.util.LinkedHashMap;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

public class BaseXMLHandler extends DefaultHandler
{
    public BaseXMLHandler()
    {
        super();
    }

    public static Map attributesToMap(Attributes attributes)
    {
        def result = new LinkedHashMap();
        def numAttributes = attributes.getLength();
        for (def i=0; i<numAttributes; ++i)
        {
            def attrname = attributes.getQName(i);
            def attrval = attributes.getValue(i);
            result[attrname] = attrval;
        }
        return result;
    }

    public static SAXParser newSAXParser()
    {
       // Create a new sax parser
       //System.setProperty("jaxp.debug", "1");
       SAXParserFactory parserFactory = SAXParserFactory.newInstance();
       parserFactory.setNamespaceAware(false);
       parserFactory.setValidating(true);        
       SAXParser parser = parserFactory.newSAXParser();
       return parser;
    }

    /**
     * Parse a String containing XML.
     * @param handler  An object whose methods will handle XML events
     * @param inputXML A string containing XML
     */
    public static void parseXML(BaseXMLHandler handler, String inputXML) 
    {
        SAXParser parser = newSAXParser();
        // parse the input xml string
        if ((inputXML == null) || (inputXML.length() == 0))
            return;
        StringReader sr = new StringReader(inputXML);          
        org.xml.sax.InputSource is = new org.xml.sax.InputSource(sr);
        parser.parse(is, handler);
    }

    /**
     * Parse a File containing XML.
     * @param handler  An object whose methods will handle XML events
     * @param inputXMLFile A File containing XML
     */
    public static void parseXML(BaseXMLHandler handler, File inputXMLFile) 
    {
        SAXParser parser = newSAXParser();
        // parse the input xml file
        Reader r = new InputStreamReader(
                        new FileInputStream(inputXMLFile), "UTF-8");
        org.xml.sax.InputSource is = new org.xml.sax.InputSource(r);
        parser.parse(is, handler);
        r.close();
    }
}
 
// end-of-file
