/*
 * Copyright (c) 2010 by SAS Institute Inc., Cary, NC 27513
 * Created on Oct. 8, 2010
 */
package com.sas.solutions.brm.xml;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import com.sas.solutions.brm.util.BRMUtil;
import com.sas.solutions.brm.xml.BaseXMLHandler;

class StatusXMLParser extends BaseXMLHandler
{
    def inConfigStatus = false;

    def configuredObjects = []; // [ [attrName: attrVal] ]

    public StatusXMLParser()
    {
        super();
    }
    
    public void startElement(String uri, String localName, String qName,
            Attributes attributes) throws SAXException 
    {
        //println("startElement: uri=\"${uri}\" localName=\"${localName}\" qName=\"${qName}\"");
        if (qName == 'ConfigStatus')
        {
            if (inConfigStatus)
                throw new Exception("Unexpected nesting of ConfigStatus elements in status.xml file");
            inConfigStatus = true;
            return;
        }
        if (qName != 'ConfiguredObject')
            throw new Exception("Unexpected tag in status.xml file: ${qName}");
        this.configuredObjects.add(attributesToMap(attributes));
    }
    
    public void endElement(String uri, String localName, String qName)
            throws SAXException 
    {
        //println("endElement: uri=\"${uri}\" localName=\"${localName}\" qName=\"${qName}\"");
        if (qName == 'ConfigStatus')
        {
            if (!inConfigStatus)
                throw new Exception("Unexpected extra ConfigStatus end tag in status.xml file");
            inConfigStatus = false;
            return;
        }
        if (qName != 'ConfiguredObject')
            throw new Exception("Unexpected end tag in status.xml file: ${qName}");
    }

    /**
     * Write the component info back out in status.xml format.
     * @param out  An object with print() and println() methods
     */
    def writeXML(out)
    {
        out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        out.println("<ConfigStatus>");
        // objMap is a LinkedHashMap preserving the original attribute order
        this.configuredObjects.each
        { objMap ->
            out.print("  <ConfiguredObject");
            objMap.each
            { attrname, attrval ->
                if (attrname == null)
                    return;
                out.print(" ${attrname}=\"${BRMUtil.escapeXML(attrval)}\"");
            }
            out.println("/>");
        }
        out.println("</ConfigStatus>");
    }

    /**
     * Write the component info back out in status.xml format.
     * @param destfile  The File to which the XML will be written
     */
    def writeXML(File destfile)
    {
        def out = new PrintWriter(
                    new OutputStreamWriter(
                        new FileOutputStream(destfile), "UTF-8"));
        try
        {
            writeXML(out);
        }
        finally
        {
            out.close();
        }
    }
    
    public static StatusXMLParser parseXML(String inputXML)
    {
        StatusXMLParser handler = new StatusXMLParser();
        parseXML(handler, inputXML);
        return handler;
    }
    
    public static StatusXMLParser parseXML(File inputXMLFile)
    {
        StatusXMLParser handler = new StatusXMLParser();
        parseXML(handler, inputXMLFile);
        return handler;
    }

    public static void test(log)
    {
        String inputXML = """<?xml version="1.0" encoding="UTF-8"?>
<ConfigStatus>
  <ConfiguredObject code="levconfig" fqid="A541RPLL.AR000001" order="wx6_planning_920_10w37"/>
  <ConfiguredObject code="metadatasrv" fqid="A541RPLL.AU000001" order="wx6_planning_920_10w37"/>
  <ConfiguredObject code="levconfig" fqid="A541RPLL.AR000002" order="wx6_planning_920_10w37"/>
  <ConfiguredObject code="levdirroot" fqid="A541RPLL.AI000003" order="wx6_planning_920_10w37"/>
  <ConfiguredObject code="bogus" order="wx6_planning_920_10w37" xyz="1"/>
  <ConfiguredObject order="wx6_planning_920_10w37" xyz="1" code="bogus"/>
  <ConfiguredObject xyz="1" code="bogus" order="wx6_planning_920_10w37"/>
</ConfigStatus>
""";
        StatusXMLParser handler = parseXML(inputXML);
        log.info("Done parsing.");
        log.info("Found ${handler.configuredObjects.size()} configured objects");
        handler.writeXML(System.out);
    }
} 

// end-of-file
