// scripts/metaquery.groovy
// Script to perform simple metadata queries

import com.sas.solutions.brm.omr.OMRConnection;

if (args.length < 2 || args[0] in ['-h', '/?', '--help'])
{
println("""Script to perform simple metadata queries.
Usage:

    brm groovy scripts/metaquery.groovy typename query [detail-level]

typename: Name of metadata type
query: XMLSelect search criteria
detail-level: "brief" or "full", default is "full"

Prints a representation of each object found and the number found.  Searches
all the metadata repositories.

Examples:

# Look up object of unknown type with Id=A541RPLL.AY000001
brm groovy scripts/metaquery.groovy Root @Id='A541RPLL.AY000001'

# Look up all TCPIPConnection objects with a Source association to a
# DeployedComponent named 'Registered SAS Application'
brm groovy scripts/metaquery.groovy TCPIPConnection "*[Source/DeployedComponent[@Name='Registered SAS Application']]"
""");
System.exit(1);
}

def typename = args[0];
def query = args[1];
def detail_level = 'full';
if (args.length > 2)
    detail_level = args[2].toLowerCase();

def installDir = System.getProperty('BRM.install.dir');
def brmConfigDir = new File(installDir, 'config');
def props = new Properties();
def propFile = new File(brmConfigDir, 'brm.properties');
println("Reading BRM config file: ${propFile}");
props.load(new FileInputStream(propFile));
def host = props.get('BRM.host.metadata');
def port = props.get('BRM.port.metadata');
def user = props.get('BRM.admin.user');
def password = props.get('BRM.admin.password');
println("Connecting to metadata server at ${host}:${port} as ${user}");

def conn = new OMRConnection(host, port, user, password);
conn.connect();

def numFound = 0;
conn.getRepositories().each
{ repos ->
    println("Searching ${repos.getName()} repository...");
    conn.setRepository(repos.getName());
    def objs = conn.doOMRQuery(typename, query, false);
    numFound += objs.size();
    objs.each
    { obj ->
        if (detail_level == 'brief')
        {
            println("${obj.getTypeName()} \"${obj.getName()}\" (${obj.getId()})");
        }
        else
        {
            println(obj.getXML());
        }
    }
}

conn.setFoundationRepository();
println("Found ${numFound} object(s) matching the query.");

conn.close();

// end-of-file
