import org.apache.log4j.Logger;

import com.sas.solutions.brm.BRMAgent;
import com.sas.solutions.brm.BRMController;
import com.sas.solutions.brm.logging.MonitoredLogger;

def printHelp()
{
    println("Stop or start web application servers");
    println("Run this command on the BRM controller machine. If the web");
    println("servers are on a different machine, make sure the BRM agent");
    println("is running on that machine.");
    println();
    println("Usage:");
    println();
    println("brm groovy scripts/webservers.groovy command SASServer1 ...");
    println();
    println("Where command is \"stop\" or \"start\"");
    println("and SASServer1 is a server name. You can add more server names.");
}

if (args.length < 2)
{
    printHelp();
    System.exit(1);
}

def command = args[0];
if (command.equalsIgnoreCase('help') || 
    command == '-h' || command == '--help' || command == '/?')
{
    printHelp();
    System.exit(0);
}

def servernames = [];
for (int i=1; i<args.length; ++i)
{
    servernames.add(args[i]);
}

Logger LOG = Logger.getLogger(BRMAgent.class);
Logger PROGRESS = Logger.getLogger("progress.agent.brm");
def msg;

def logger = new MonitoredLogger(PROGRESS, LOG);
def controller = new BRMController();
def webhost = controller.getConfigProp('BRM.host.webapp');
if (webhost == null)
{
    logger.error("Missing BRM property BRM.host.webapp");
    System.exit(1);
}

if (command.equalsIgnoreCase('start'))
{
    servernames.each
    { serverName ->
        PROGRESS.info("Starting server: " + serverName);
        controller.setProperty('BRM.webserver.name', serverName);
        controller.runTarget(webhost, '', 'startwebserver');
    }
    System.exit(0);
}
else if (command.equalsIgnoreCase('stop'))
{
    servernames.each
    { serverName ->
        PROGRESS.info("Stopping server: " + serverName);
        controller.setProperty('BRM.webserver.name', serverName);
        try
        {
            controller.runTarget(webhost, '', 'stopwebserver');
        }
        catch (com.sas.solutions.brm.errors.BRMAntError e)
        {
            println(e);
        }
    }
    System.exit(0);
}
else
{
    PROGRESS.error("Unknown command: ${command}");
    System.exit(1);
}

// end-of-file
