// webservice.groovy

import org.apache.log4j.Logger;
import com.sas.solutions.brm.Constants;
import com.sas.solutions.brm.util.BRMUtil;
import com.sas.solutions.brm.webservice.WSMWrapper;

def printHelp()
{
    println("SAS BI WebService Utility");
    println("Use:");
    println("brm groovy scripts/webservice.groovy command [parameter...]");
    println();
    println("Commands:");
    println("  list               -- Print names of registered web services");
    println("  remove servicename -- Remove a web service");
}

// Main method.
// Return 0 if successful, some non-zero integer otherwise
// The return value is passed to System.exit()
def main()
{
    def filteredArgs = BRMUtil.filterCmdLineProps(args);
    if (filteredArgs.length < 1)
    {
        printHelp();
        return 0;
    }

    def command = filteredArgs[0];

    if (command.equalsIgnoreCase('help') || 
        command == '-h' || command == '--help' || command == '/?')
    {
        printHelp();
        return 0;
    }

    def log = Logger.getLogger('com.sas.solutions.brm.scripts.webservice');
    log.info("Running BRM web service controller script");

    def controlProps = new Properties();
    BRMUtil.loadPropFileAsResource(Constants.BRM_CONFIG_FILE, controlProps, 
                                   null);

    def getPropOrDie = { propname ->
        def propval = controlProps.getProperty(propname);
        if (propval == null)
        {
            throw new Exception("Missing property: ${propname}");
        }
        return propval;
    }

    def metadataHost = getPropOrDie('BRM.host.metadata');
    def metadataPort = getPropOrDie('BRM.port.metadata');
    def username = getPropOrDie('BRM.admin.user');
    def password = getPropOrDie('BRM.admin.password');

    def wsm = null;
    def wsmUrl = System.getProperty('WebServiceMaker.url');
    if (wsmUrl == null)
    {
        println("Connecting to metadata server ${metadataHost}:${metadataPort} as user ${username}");
        wsm = new WSMWrapper(log, metadataHost, metadataPort, username, 
                             password);
    }
    else
    {
        println("Connecting to WebServiceMaker at URL: ${wsmUrl}");
        wsm = new WSMWrapper(log, wsmUrl);
    }

    if (command.equalsIgnoreCase('list'))
    {
        def webServices = wsm.listWebServices(username, password);
        webServices.each
        { webService ->
            println(webService);
        }
    }
    else if (command.equalsIgnoreCase('remove'))
    {
        if (filteredArgs.length < 2)
        {
            println("Missing web service name");
            println("Use \"help\" command for more information.");
            return 1;
        }
        def serviceName = filteredArgs[1];
        println("Removing web service: ${serviceName}");
        wsm.removeWebService(serviceName, username, password);
    }
    else
    {
        println("Unknown command: ${command}");
        return 1;
    }
    return 0;
}

System.exit(main());

// end-of-file
